function jacobian_indexAll = makeJacobianIndex

jacobian_indexAll = cell(1,1);

x = ones(47,1)*0.5;

%% Prepare flux for Substrate
flux = zeros(30);

flux(2,1)  = x(12);                   % rapid to HE1
flux(18,1) = x(19);                   % rapid to urine

flux(14:16,1) = x(13:15);             % rapid to distribution
flux(1,14:16) = x(13:15)./x(16:18);   % distribution to rapid

flux(2,12)  = x(2);                   % intestine to HE1
flux(17,12) = ((1-x(21))/x(21))*x(2); % intestine to feces

flux(20,19) = x(22);                  % transit 1 to 2
flux(21,20) = x(22);                  % transit 2 to 3
flux(12,21) = x(22);                  % transit 3 to intestine

fBPSinf  = x(4) * 0.2;
fHPSeff  = x(5) * 0.2;
fHCLint  = x(6) * 0.2;
fHCLbile = fHCLint      /(1+x(23));
fHCLmet  = fHCLint*x(23)/(1+x(23));
fluxHEHC = [0, fHPSeff; fBPSinf, -fHCLmet];

flux(2:3,  2 :3 ) = fluxHEHC;
flux(4:5,  4 :5 ) = fluxHEHC;
flux(6:7,  6 :7 ) = fluxHEHC;
flux(8:9,  8 :9 ) = fluxHEHC;
flux(10:11,10:11) = fluxHEHC;
flux(4 ,2 ) = x(12);
flux(6 ,4 ) = x(12);
flux(8 ,6 ) = x(12); 
flux(10,8 ) = x(12);
flux(1 ,10) = x(12); % HE5 to rapid
flux(19,[3,5,7,9,11]) = fHCLbile; 

fluxCont = flux;


%% Prepare flux for Inhibitor
flux = zeros(30);

flux(2 ,1 ) = x(12);                   % rapid to HE1
flux(3 ,2 ) = x(12);
flux(4 ,3 ) = x(12);
flux(5 ,4 ) = x(12); 
flux(6 ,5 ) = x(12);
flux(1 ,6 ) = x(12);                  % HE5 to rapid

flux(9:11,1) = x(13:15);              % rapid to distribution_blood
flux(1,9:11) = x(13:15);              % distribution_blood to rapid

flux(9:11,12:14) = diag(x(43)*x(40:42)*x(36));           % cells to blood 
flux(12:14,9:11) = diag(x(43)*x(40:42)*x(44)./x(37:39)); % blood to cells

flux(9:11,1) = x(13:15);              % rapid to distribution_cells
flux(1,9:11) = x(13:15);              % distribution_cells to rapid

flux(2,7)   = x(32);                  % intestine to HE1
flux(15,7)  = ((1-x(45))/x(45))*x(32);% intestine to feces

flux(19,[3,5,7,9,11]) = x(34) * 0.2; 

fluxInhi = flux;


%% Combine flux
z = zeros(30,30);

flux = [fluxCont,z,z; z,fluxCont,z; z,z,fluxInhi];



% Put -(sum(outflow)) at flux(k,k) to keep mass balance
fluxDiag=sum(flux - diag(diag(flux)),1);
flux = flux - diag(fluxDiag);

jacobian_index = flux ./ flux;
jacobian_index(isnan(jacobian_index)) = 0;

jacobian_indexAll{1} = sparse(jacobian_index);

end

