function jacobian_indexAll = makeJacobianIndex

jacobian_indexAll = cell(1,1);


x = ones(110,1);



%% Parent
flux = zeros(10);

flux(3,1) = x(1);      % Rapid to Liver
flux(1,3) = x(1)/x(8); % Liver to Rapid
flux(4,1) = x(9);      % Rapid to Peripheral
flux(1,4) = x(10);     % Peripheral to Rapid
flux(5,1) = x(11);     % Rapid to Urine

kIntesElim = x(5) * (1-x(7))/ x(7);

flux(2,6) = x(6);      % Intes Transit to Intes
flux(3,2) = x(5);      % Intes to Liver
flux(2,2) =-kIntesElim / (1+sum(x(12:14))); % Intes to Feces

fluxPare = flux;



%% Metabolites

fluxMet1 = metaboliteFlux(x(41:50),x(1));
fluxMet2 = metaboliteFlux(x(51:60),x(1));
fluxMet3 = metaboliteFlux(x(61:70),x(1));



%% Inhibitors

fluxInhi1 = inhibitorFlux(x( 91:100),x(1));
fluxInhi2 = inhibitorFlux(x(101:110),x(1));


%% Combine flux

z = zeros(10);

fluxCon = [...
    fluxPare, z,z,z,z,z;...
    z,fluxMet1, z,z,z,z;...
    z,z,fluxMet2, z,z,z;...
    z,z,z,fluxMet3, z,z;...
    z,z,z,z,fluxInhi1,z;...
    z,z,z,z,z,fluxInhi2;...
    ];

% Add intestinal metabolism
kIntesArr    = x(12:15) * kIntesElim / (1+sum(x(12:15)));
kIntesArr(4) = -kIntesArr(4);
fluxCon([12,22,32,2],2) = kIntesArr;% Intes to Met 1-3 Intes



%% Add metabolisms

fluxCon = addMetabolism(fluxCon,x);



%% Combine and Put -(sum(outflow)) at flux(k,k) to keep mass balance
flux = fluxCon;

fluxDiag=sum(flux - diag(diag(flux)),1);
flux = flux - diag(fluxDiag);




jacobian_index = flux ./ flux;
jacobian_index(isnan(jacobian_index)) = 0;

jacobian_indexAll{1} = sparse(jacobian_index);

end



function flux = addMetabolism(flux,x)

flux(13,3) = sum(x(16:19))/x(8); % Liver to Met 1 Liver
flux(23,3) = sum(x(20:23))/x(8); % Liver to Met 2 Liver
flux(33,3) = sum(x(24:27))/x(8); % Liver to Met 3 Liver
flux( 3,3) =-sum(x(28:31))/x(8); % Liver to Met other

flux(13,13)=-sum(x(47:50))/x(43);% Met 1 Hepatic elimination
flux(23,23)=-sum(x(57:60))/x(53);% Met 2 Hepatic elimination
flux(33,33)=-sum(x(67:70))/x(63);% Met 3 Hepatic elimination

end



function flux = metaboliteFlux(x,qh)

flux = zeros(10);


flux(3,2) = x(2);     % Intes to Liver

flux(3,1) = qh;       % Rapid to Liver
flux(4,1) = x(4);     % Rapid to Peripheral
flux(5,1) = x(6);     % Rapid to Urine

flux(1,3) = x(1)/x(3);% Liver to Rapid
flux(1,4) = x(5);     % Peripheral to Rapid

end




function flux = inhibitorFlux(x,qh)

flux = zeros(10);

flux(3,1) = qh;       % Rapid to Liver
flux(1,3) = qh/x(5);  % Liver to Rapid
flux(3,3) =-x(9)/x(5);% Liver to elimination
flux(4,1) = x(6);     % Rapid to Peripheral
flux(1,4) = x(7);     % Peripheral to Rapid
flux(5,1) = x(8);     % Rapid to Urine

flux(2,6) = x(3);     % Intes Transit to Intes
flux(3,2) = x(2);     % Intes to Liver
flux(2,2) =-x(2) * (1-x(4))/ x(4) ;% Intes to Feces

end


