function h = createFigLM1to11(folder)
%createFigLM1to11('/Users/rokuninme/Dropbox/0-inv_prob_yoshi/resultsLM/201303/PRVRIF_130327_1to11/20130327-1125-Rfree_EHC')
% FigSet,uFig,isSortSS,num_disp
% h = createFigLM(FigSet,uFig,1)
% isSortSS == 1 -  just sort samples
% isSortSS == 2 -  erase samples with similar parameter values

origDir = cd;
cd(folder);

try
for k1 = 1:6
    %% Load and Setting
    
    [uFig,FigSet] = loadFile(k1);

    saveSS = FigSet.saveSS;
    x_out  = FigSet.x_out;
    
    Numbers.actTimeCourse = FigSet.actTimeCourse;
    Numbers.timeFitComp   = FigSet.timeFitComp;
    Numbers.timePoints    = FigSet.timePoints;
    Numbers.rangedisplay  = FigSet.rangedisplay;
    
    timePoints2 = Numbers.timePoints; timePoints2(1) = [];
    
    subPind = mod(k1+1,2)+1;
    if k1 == 6, secondCmp = 1; else secondCmp = 0; end
    
    %% Sort with SS
    
    [uFig,ssIndex] = selectSample(x_out,saveSS,uFig);
    parDivDisp = numel(ssIndex);
    if k1==2 || k1==4, ssIndex = prevssIndex(ssIndex); end
    prevssIndex = ssIndex;
    
    
    %% Figure preparation
    legendLabel = cell(parDivDisp + 1,1);
    for k = 1:parDivDisp
        legendLabel{k} = ['#',num2str(ssIndex(k))];
    end
    legendLabel{parDivDisp+1} = 'Obs.';
    
    %% Reorder samples
    % Ref: Reorder legend entires
    % http://www.mathworks.se/matlabcentral/newsreader/view_thread/159423
    
    lsGroupNum = 4; lsSampNum = ceil((parDivDisp)/lsGroupNum); forFigIndInv = [];
    for k = 1:lsSampNum
        forFigIndInv = [forFigIndInv, k:lsSampNum:parDivDisp]; %#ok<AGROW>
    end
    
    forFigIndInv = forFigIndInv(1:parDivDisp);
    [temp,forFigInd] = sort(forFigIndInv); %#ok<ASGLU>
    
    for k2 = 1:Numbers.timeFitComp
        uFig(:,:,k2) = uFig(:,forFigInd,k2);
    end
    
    
    %% Create figure
    h = figure(floor((k1+1)/2));
    
    S = get(0, 'ScreenSize');
    lineColorMap = winter(ceil((parDivDisp)/4));
    set(h,'position', [S(3)-950 100 1000 500]);
    set(0, 'DefaultAxesColorOrder', lineColorMap,...
        'DefaultAxesLineStyleOrder', '-|--|-.|:')
    
    subplot(1,2,subPind);
    
    uFigEach = uFig(:,:,1+secondCmp);
    semilogy(Numbers.timePoints, uFigEach,'LineWidth', 1.5);
    hold on

    actTimeCourse = Numbers.actTimeCourse(:,2+secondCmp);
    actTimeCourse(1) = [];
    plot(timePoints2, actTimeCourse, 'mo', ...
        'MarkerEdgeColor', [1,0,0], 'MarkerFaceColor', [1,0.5,0],...
        'LineWidth', 1.2, 'MarkerSize', 10);
    
    % [left bottom width height]
    if subPind == 1
        set(gca,'Position',[0.05, 0.05, 0.9/2-0.07, 0.9]);
    else
        set(gca,'Position',[0.52, 0.05, 0.9/2-0.07, 0.9]);
    end
    set(gca,'FontSize',20);
    
    if 2-subPind == 0
        set(gca,'yticklabel',[]);
    end
    if k1 ~= 5
        c = get(gca, 'Children');
        c = c(end:-1:1);
        hleg = legend(c([forFigIndInv,end]),legendLabel);
        set(hleg,'FontSize',12);
        pos = get(hleg,'position');
        if subPind == 1
            set(hleg,'Position',[0.43,pos(2:4)]);
        else
            set(hleg,'Position',[0.9 ,pos(2:4)]);
        end
    end
    
    
    xlim(Numbers.xlim)
    ylim(Numbers.ylim)
    
    hold off
end

for k1 = 1:3
    hgexport(figure(k1), ['Fig',num2str(k1),'.eps'])
    saveas(  figure(k1), ['Fig',num2str(k1),'.fig'])
end
catch err
    cd(origDir);
    rethrow(err);
end

cd(origDir);

end




function [uFig,ssInd] = selectSample(x_out,SS,uFig)

ssInd = 1:numel(SS);

x_out = x_out(:,SS<(min(SS)*10));
uFig  = uFig(:, SS<(min(SS)*10),:);
ssInd = ssInd(SS<min(SS)*10);
SS    = SS(SS<min(SS)*10);

num_x = size(x_out,1);
k = 1;
while k <= size(x_out,2)
    xoNorm = zeros(size(x_out));
    for k1 = 1:size(x_out,2)
        xoNorm(:,k1) = x_out(:,k1) ./ x_out(:,k);
    end
    xoNorm(isnan(xoNorm)) = 1;
    xoNorm(xoNorm == Inf) = 0;
    tenPerDiff = (sum((0.9<=xoNorm).*(xoNorm<=1.1),1) == num_x);
    notSSmin = (SS > min(SS(tenPerDiff)));
    
    eracceInd = tenPerDiff .* notSSmin;
    x_out(:, eracceInd==1)  = [];
    uFig(:,  eracceInd==1,:)= [];
    ssInd(:, eracceInd==1)  = [];
    SS(eracceInd==1)        = [];
    k=k+1;
end


end





function [uFig,FigSet] = loadFile(k1) %#ok<STOUT>

switch k1
    case 1
        load('./4/uFig.mat')
        load('./4/FigSet.mat')
    case 2
        load('./5/uFig.mat')
        load('./5/FigSet.mat')
    case 3
        load('./9/uFig.mat')
        load('./9/FigSet.mat')
    case 4
        load('./10/uFig.mat')
        load('./10/FigSet.mat')
    case {5,6}
        load('./11/uFig.mat')
        load('./11/FigSet.mat')
end

end