function parDistCmp(initXfree, finalXfree, normXfree,...
    toLog10, isNormalize, isAddOrig, ylimRange, yellowNumFin)
% PARDISTCMP visualize the original and final parameter distribusions
%
% parDistCmp(initXfree, finalXfree) creates the figure showing the
% distribution of each optimized parameter value.
%
% parDistCmp(initXfree, finalXfree, [], isLog) creates the figure showing the
% distribution of each optimized parameter value in log10 scale.
%
% parDistCmp(initXfree, finalXfree, [], isLog, isNormalize) normalize the
% optimized parameter values with the range of InitX, if isNormalize ==
% 1 (default: isNormalize = 0). If normalized values are equal to 1/-1,
% they correspond to the maximum/minimum value of initial parameter
% distributions.
%
% parDistCmp(initXfree, finalXfree, [], isLog, isNormalize, isAddOrig) add subplot
% showing the distribution of initial parameter values, if isAddOrig == 1|2,
% or overlay if isAddOrig == 3, (default: 0).
%
% parDistCmp(initXfree, finalXfree, [], isLog, isNormalize, isAddOrig, ylimRange)
% define the figure ylim range manually.
%
%
% Example:
%   parDistCmp(X(:,:,1),X(:,:,end),[],1,1,0)
%
% My specific case example 1:
%   finalX = FinalXsorted(:,FinalXsorted(1,:,1)~=FinalXsorted(2,:,1))'; 
%   parDistCmp(exp(X(:,:,1)),finalX,[],1,1,0)
%   set(gcf,'Position',[20,20,640,300]);
%
% My specific case example 1-2:
%   finalX = FinalXsorted(:,FinalXsorted(1,:,1)~=FinalXsorted(2,:,1))';
%   InitialX = InitialX';
%   iniX = InitialX(:,InitialX(1,:)~=InitialX(2,:))';
%   parDistCmp(iniX,finalX,[],1,1,0)
%   set(gcf,'Position',[20,20,720,300]);
%
% My specific case example2:
%   for k = 1:3000
%     ss(k) = (Y(:,k,end) - ystar)'*(Y(:,k,end) - ystar);
%   end
%   [a b] = sort(ss);
%   Xsort = X(:,b,20);
%   parDistCmp(Xsort([1:31],:),Xsort([32 34:63],:),[],0,1,0)
%
% My specific case example 3:
%   parDistCmp(finalXwoM1, finalX([1:11 20:25],:),[], 1,1,0)
%   parDistCmp(finalXO, finalXB([1:15 21:end-1],:),[], 1,1,0)
%   parDistCmp(finalXO, finalXB([1:end-1],:),[], 1,1,3)
%
% My specific case example 4:
%   finalX = FinalXsorted(:,FinalXsorted(1,:,1)~=FinalXsorted(2,:,1))';
%   parDistCmp(log(finalX), log(finalX(:,1:10)),X(:,:,1), 0,1,3,[-5,5])
%
% Misc
%   parDistCmp(Xo(:,:,end),Xb(1:55,:,end),Xo(1:55,:,1),0,1,3)
%   parDistCmp(log(finalXO), log(finalXB([1:end-1],:)),X(1:55,:,1), 0,1,3)
%   parDistCmp(log(finalXO(:,1:3)), log(finalXB([1:end-1],1:3)),X(1:55,:,1), 0,1,3, [-5,5])
%
% RPG case -> 20130118-1642- , 20130117-1504-
%   parDistCmp(log(finalXO([1:11,13:end],:)), log(finalXB([1:11,13:end],:)),X([1:11,13:end],:,1), 0,1,3,[-3,3])


global xDispInt

%% Initialize

xDispInt = 1;

if nargin == 2 || numel(normXfree) == 0
    normXfree = initXfree;
end

if nargin <= 3 || numel(toLog10)     == 0, toLog10     = 0; end
if nargin <= 4 || numel(isNormalize) == 0, isNormalize = 0; end
if nargin <= 5 || numel(isAddOrig)   == 0, isAddOrig   = 0; end
if nargin <= 7 || numel(yellowNumFin)== 0, yellowNumFin= 10;end
if nargin <= 6 || numel(ylimRange)   == 0
    autoYlim    = 1; 
elseif  numel(ylimRange) == 3
    autoYlim = 2; ylimRangeIn = ylimRange(1:2);
else
    autoYlim = 0;
end


if toLog10
    initXfree  = log10(initXfree)';
    finalXfree = log10(finalXfree)';
    normXfree  = log10(normXfree)';
    isLog10Str = 'log10';
else
    initXfree  = initXfree';
    finalXfree = finalXfree';
    normXfree  = normXfree';
    isLog10Str = '';
end

initSampNum = size(finalXfree,1); %#ok<NASGU>
finalSampNum = size(finalXfree,1); %#ok<NASGU>
freeNum = size(initXfree,2);
if freeNum ~= size(finalXfree,2)
    error('Numbers of free parameters do not match')
end


%% Settings

axLabelFontSize= 16;
axFontSize     = 20;
titleFontSize  = 20;
yellowNumInit  = 0;


%% Normalize values

if isNormalize == 1;
    [initXfree,finalXfree]=normalizeFinalX(initXfree,finalXfree,normXfree);
    ylabelStr = ['normalized ' isLog10Str ' parameter values'];
else
    ylabelStr = [isLog10Str ' parameter values'];
end


%% Original distribution
figure(6);
set(gcf, 'Renderer', 'zbuffer')

switch isAddOrig
    case {1,2}
        
        subplot(isAddOrig,3-isAddOrig,1);
        
        initXplot=initXfree(yellowNumInit+1:end,:);
        plot(1:freeNum,initXplot,'Color',[0,0,1],'linewidth',1)
        %plot(1:freeNum,initXplot,'linewidth',1)
        hold on
        if yellowNumFin ~= 0
            initXplot=initXfree(1:yellowNumInit,:);
            plot(1:freeNum,initXplot,'Color',[1,1,0],'linewidth',1)
        end
        
        set(gca,'FontSize',axFontSize);
        %xlabel('#parameters ','FontSize',axLabelFontSize)
        %title('Original','FontSize',titleFontSize)
        
        xlim([0.5 freeNum+0.5])
        if autoYlim
            ylimRange1 = ylim;
        end
        
    otherwise
        
        set(gca,'FontSize',axFontSize);
end

%% Fitted distribution

switch isAddOrig
    case {0,1,2}
        
        if isAddOrig ~= 0, subplot(isAddOrig,3-isAddOrig,2), end;
        
        %{
        % Gradation
        colVec = (1./(1:finalSampNum).^0.3)';
        colVec = (colVec-colVec(end))/(1-colVec(end)); %Normalization
        colVec = [colVec colVec 1-colVec];

        for k = finalSampNum:-1:1
            plot(1:freeNum,finalXfree(k,:),'Color',colVec(k,:),'linewidth',1)
            hold on
        end
        %}
        
        finalXplot=finalXfree(yellowNumFin+1:end,:);
        h = plot(1:freeNum,finalXplot','Color',[0,0,1],'linewidth',1); % Blue
        %h = plot(1:freeNum,finalXplot','linewidth',1); % Mix
        hold on
        if yellowNumFin ~= 0
            finalXplot=finalXfree(1:yellowNumFin,:);
            h = plot(1:freeNum,finalXplot','Color',[1,1,0],'linewidth',1); % Yellow
        end
        set(gca,'FontSize',axFontSize);
        
    case 3
        
        initXrange = [min(initXfree);max(initXfree)]';
        finalXrange= [min(finalXfree);max(finalXfree)]';
        h = plotCombXrange(initXrange,finalXrange,freeNum,128);

        
end

hold off
%{
set(gca,'FontSize',axFontSize);
xlabel('#parameters ','FontSize',axLabelFontSize)
ylabel(ylabelStr,'FontSize',axLabelFontSize)
title('Optimized','FontSize',titleFontSize)
%}

%% Set axis properties

if autoYlim
    switch isAddOrig
        case {0,3}
            ylimRange = ylim;
        case {1,2}
            ylimRange2 = ylim;
            ylimRange = ...
                [min(ylimRange1(1),ylimRange2(1)) max(ylimRange1(2),ylimRange2(2))];
    end
end
if autoYlim == 2
    ylimRange = ...
        [max(ylimRange(1),ylimRangeIn(1)) min(ylimRange(2),ylimRangeIn(2))];
end
setAxis(freeNum,isAddOrig,ylimRange,isNormalize,h)

set(gcf,'Position',[20,20,1280,600]);

end




function [initXfree,finalXfree]=normalizeFinalX(initXfree,finalXfree,normXfree)

normXfreeMean  = (max(normXfree) + min(normXfree))/2;
normXfreeRange =  max(normXfree) - min(normXfree);
normXfreeRange = diag(1./normXfreeRange);

%{
size(initXfree)
size(finalXfree)
size(normXfree)
%}

initXfree = ...
    (initXfree - repmat(normXfreeMean,size(initXfree,1),1))...
    *normXfreeRange*2;
finalXfree = ...
    (finalXfree - repmat(normXfreeMean,size(finalXfree,1),1))...
    *normXfreeRange*2;

%{
initXfreeMean  = (max(initXfree) + min(initXfree))/2;
initXfreeRange =  max(initXfree) - min(initXfree);
initXfreeRange = diag(1./initXfreeRange);
initXfree = ...
    (initXfree - repmat(initXfreeMean,size(initXfree,1),1))...
    *initXfreeRange*2;
finalXfree = ...
    (finalXfree - repmat(initXfreeMean,size(finalXfree,1),1))...
    *initXfreeRange*2;
%}

end





function h = plotCombXrange(initXrange,finalXrange,freeNum,xdiv)



TEST = 0;
if TEST  
    xdiv = 128; %#ok<UNRCH>
    freeNum = 5;
    initXrange  = [[-1 -2 -1  -1 -2]' [ 2 -1  0    0  0]' ];
    finalXrange = [[ 3 -4 -3  -2 -2]' [ 4 -3 -1.5 -1 -1]' ];
    
end


dx = 1/xdiv;

initXmin  = initXrange(:,1);
initXmax  = initXrange(:,2);
finalXmin = finalXrange(:,1);
finalXmax = finalXrange(:,2);
xrangeTmp = [initXmin,initXmax,finalXmin,finalXmax];

xrange        = zeros((freeNum-1)*xdiv+1,6);
xVec = xrangeTmp(1,:); 
xrange(1,1:4) = xVec;
xrange(1,5:6) = [max(xVec([1,3])), min(xVec([2,3]))]; 

for k1 = 1:freeNum-1
    for k2 = 1:xdiv
        xVec = ...
            (xrangeTmp(k1,:)*(xdiv-k2) + xrangeTmp(k1+1,:)*k2)/xdiv;
        
        
        if xVec(4) < xVec(1)
            xVec(5) = xVec(4);
        else
            xVec(5) = max(xVec(1),xVec(3));
        end
            
            
        if xVec(3) > xVec(2)
            xVec(6) = xVec(3);
        else
            xVec(6) = min(xVec(2),xVec(4));
        end
        
        xrange((k1-1)*xdiv+k2+1,:) = xVec;
        
    end
end

xrange = [xrange(:,1),xrange(:,2)-xrange(:,1),...
    xrange(:,3)-xrange(:,2),xrange(:,4)-xrange(:,3),...
    xrange(:,5)-xrange(:,4),xrange(:,6)-xrange(:,5)];

h = area(1+0:dx:freeNum,xrange,'LineStyle','none');
grid on
set(h(1),'Visible','off')
set(h(3),'Visible','off')
set(h(5),'Visible','off')
set(h(2),'FaceColor',[0 65 255]/255)
set(h(4),'FaceColor',[53 161 107]/255)
set(h(6),'FaceColor',[255 153 0]/255)
%set(gca,'Layer','top')
set(h,'LineStyle','none')



end



function setAxis(freeNum,isAddOrig,ylimRange,isNormalize,h)
global xDispInt

xlim([0.5 freeNum+0.5])
switch isAddOrig
    case 0
        ylim(ylimRange); grid on;
        if isNormalize, addytick(1);addytick(-1); end
        set(gca,'xtick',unique([1:xDispInt:freeNum,freeNum]));
    case {1,2}
        
        subplot(isAddOrig,3-isAddOrig,2); ylim(ylimRange); grid on;
        if isNormalize, addytick(1);addytick(-1); end
        set(gca,'xtick',unique([1:xDispInt:freeNum,freeNum]));
        
        subplot(isAddOrig,3-isAddOrig,1); ylim(ylimRange); grid on;
        if isNormalize, addytick(1);addytick(-1); end
        set(gca,'xtick',unique([1:xDispInt:freeNum,freeNum]));
    case 3
        
        ylim(ylimRange); grid on;
        if isNormalize, addytick(1);addytick(-1); end
        set(gca,'xtick',unique([1:xDispInt:freeNum,freeNum]));
        
        set(h,'BaseValue',ylimRange(1));
end


end



function addytick( tk, tklbl )
%ADDXTICK --- Add new tick to x-axis
%   Ex.    addxtick(3.14)
%   Ex2.  addxtick(3.14, 'Pi')
%
% http://ichiro-maruta.blogspot.jp/2009/01/matlab.html

if nargin==1
    set(gca,'YTick',unique(sort([get(gca,'YTick'), tk ] )));
else
    temp = cellstr(get(gca,'YTickLabel'));
    % temp = {temp{:},tklbl};
    % If A is a cell array with a single row, replace { A{:} X } with [A {X}]
    temp = [temp {tklbl}];
    [temp2,idx] = sort([get(gca,'YTick'),tk]);
    [temp2,idx2]=unique(temp2);
    idx=idx(idx2);
    set(gca,'YTick',temp2);
    set(gca,'YTickLabel',temp(idx));
end

end
