function [x_out,x_stdev,x_outTF,x_stdevTF,t,u,AIC,saveSS,statOpt] = ...
    LMmethod(resultFolder,InitialXY,Numbers,ForODE,...
    ParameterIndex,divideNumberProd,xfree,Model) %#ok<INUSL>

global Numbers2 ForODE2 ParameterIndex2 InitialXY2 Model2
Numbers2 = Numbers;
ForODE2 = ForODE;
Model2 = Model;
ParameterIndex2 = ParameterIndex;
InitialXY2 = InitialXY;

statOpt.stat = 1;
timeLM = tic; 

%% Fitting

options = optimset('Algorithm',ForODE.algorithm,'TolX',1e-6,'TolFun',1e-6,...
    'Display','off','FunValCheck','off','outputfcn',@outfun);

try
    [xFit,resnorm,residual,exitflag,output,lambda,jacobian] = ...
        lsqnonlin(@PBPK_model_LM,xfree,[],[],options); %#ok<ASGLU>
    
    switch exitflag
        case {1,2,3,4}
            [x_out,x_stdev,x_outTF,x_stdevTF] = ...
                parSDTF(residual',jacobian,InitialXY,ParameterIndex,...
                xFit,Numbers.input);
            
            % Output
            [t,u] = recalcTimeProfile(Numbers,ForODE,x_out);
            [AIC,saveSS] = calcAIC(u,Numbers2,ForODE2);
            
        case {-1,-3}
            [x_out,x_stdev,x_outTF,x_stdevTF,t,u,AIC,saveSS] =  errOut;
            statOpt.stat = exitflag;
            
        otherwise
            error(['Error: exitflag = ',num2str(exitflag),' at line 49 in @LMmethod'])
            
    end
    statOpt.flag = exitflag;
    
    
catch err
    
    disp([char(10),err.identifier]);
    disp([err.message,char(10)]);
    for k=1:numel(err.stack)
        disp([err.stack(k).name,' at line ',num2str(err.stack(k).line)]);
    end
    disp(char(10));
    [x_out,x_stdev,x_outTF,x_stdevTF,t,u,AIC,saveSS] =  errOut;  
    statOpt.stat = 0;
    
end




    function stop = outfun(x,optimValues,state) %#ok<INUSL>
        stop = false;
        
        switch state
            case 'init'
                %figure(2)
                %hold on
                
            case 'interrupt'
                %Not displayed in lsqnonlin (?)
                
            case 'iter'
                % Concatenate current point and objective function
                % value with history. x must be a row vector.
                
                %history.fval = [history.fval; optimValues.fval];
                
                %disp(['iter / funcc = ', num2str(optimValues.iteration),...
                %    ' / ', num2str(optimValues.funccount),...
                %    ' after ',num2str(toc(timeLM)),' sec'])                
                
                if toc(timeLM) > 1200
                    statOpt.timeLM    = toc(timeLM);
                    statOpt.iteration = optimValues.iteration;
                    statOpt.funccount = optimValues.funccount;
                    stop = true;
                end
                
                % Concatenate current search direction with
                % searchdir.
                % searchdir = [searchdir;...
                %     optimValues.searchdirection'];
                
                % Plot and label points with iteration number.
                % Add .15 to x(1) to separate label from plotted 'o'
                %plot(x(1),x(2),'o');
                %text(x(1)+.15,x(2),num2str(optimValues.iteration));
            case 'done'
                %hold off
            otherwise
        end
    end




    function [x_out,x_stdev,x_outTF,x_stdevTF,t,u,AIC,saveSS] =  errOut
        
        x_out(ParameterIndex.fix)  = InitialXY.xfix;
        x_out(ParameterIndex.free) = Inf;
        x_stdev = ones(1,Numbers.input) * Inf;
        
        x_outTF(ParameterIndex.fix)  = InitialXY.xfix;
        x_outTF(ParameterIndex.free) = Inf;
        x_stdevTF = ones(1,Numbers.input) * Inf;
        
        t = zeros(size(Numbers.timePoints));
        u = zeros(size(Numbers.timePoints,2),sum(Numbers.compAll));
        
        AIC = Inf;
        saveSS = Inf;
        
    end

end




function [x_out,x_stdevOut,x_outTF,x_stdevTF]=parSDTF(v,J,InitialXY,ParameterIndex,xFit,num_input)

%% Calculate SD
% http://www.mathworks.com/matlabcentral/newsreader/view_thread/314454

[rows,cols]=size(J);

% A posteriori variance factor
Sigma_o = sqrt((v'*v) / (rows-cols));
if rows-cols<= 0
    warning(['Model is overparameterized: ' ...
        'The number of parameters should not exceed '...
        'that of the data to optimize.'])  %#ok<WNTAG>
end

% Unscaled Covariance Matrix (Inverse of the Normal Equation matrix)
%Q_xx2 = inv(J'*J); %#ok<NASGU>
Q_xx = pinv(J'*J);

% Precision measures ,i.e. the standard deviation of each parameter
diagonals_of_Q_xx = diag(Q_xx)';
x_stdev = Sigma_o .* sqrt(diagonals_of_Q_xx);
%x_stdev

%% Revert to original order

freelimits = InitialXY.freelimits;
pindExp = ParameterIndex.exp;
pindSin = ParameterIndex.sin;

xFitOrig = xFit;
xFitOrig(pindExp) = exp(xFitOrig(pindExp))+freelimits(1,pindExp);
%xFitOrig(pindSin) = (sin(xFitOrig(pindSin))+1)./2.*...
%    (freelimits(2,pindSin)-freelimits(1,pindSin))+...
%    freelimits(1,pindSin);
limPlus = freelimits(2,pindSin) + freelimits(1,pindSin);
limMinus= freelimits(2,pindSin) - freelimits(1,pindSin);
xFitOrig(pindSin) = (limPlus + limMinus .* sin(xFitOrig(pindSin)))/2;


%% Delta method
%xFit(pindExp)
%exp(xFit(pindExp))
%x_stdev(pindExp)
x_stdevDelta = x_stdev;

x_stdevDelta(pindExp) = exp(xFit(pindExp)) .* x_stdev(pindExp);
%x_stdevDelta(pindSin) = (freelimits(2,pindSin)-freelimits(1,pindSin))/2 ...
%    .* cos(xFit(pindSin)) .* x_stdev(pindSin);
x_stdevDelta(pindSin) = abs(limMinus .* cos(xFit(pindSin)) .* x_stdev(pindSin)) / 2;

%% Parameter output
x_out(ParameterIndex.fix)  = InitialXY.xfix;
x_out(ParameterIndex.free) = xFitOrig;
x_stdevOut = zeros(1,num_input);
x_stdevOut(ParameterIndex.free)  = x_stdevDelta;

x_outTF(ParameterIndex.fix)  = InitialXY.xfix;
x_outTF(ParameterIndex.free) = xFit;
x_stdevTF = zeros(1,num_input);
x_stdevTF(ParameterIndex.free)  = x_stdev;

end




function [t,u] = recalcTimeProfile(Numbers,ForODE,x_out)

u = [];

for conditionNumber = 1:Numbers.sepCond
    switch char(ForODE.calcAlgorithm(1))
        case 'ode_mfile'
            if size(x_out,1) == numel(x_out), x_out = x_out'; end
            
            cd('solveODE')
            
            mfunc = ForODE.mfuncAll{conditionNumber};
            
            for k1 = 1:numel(ForODE.odeSolvers)
                [t,uCalc] = mfunc(ForODE.odeSolvers{k1}, x_out, ...
                    Numbers.timePoints, ForODE.jacobian_indexAll{conditionNumber}, ...
                    Numbers.compAll(conditionNumber),...
                    Numbers.doseSin,Numbers.doseInf);
                if t(end)==Numbers.timePoints(end), break; end
            end
            
            cd('../')
            
        case 'FILT'
            cd('solveFILT')
            [t,uCalc] = solveFILT(x_out, Numbers, ForODE, conditionNumber);
            cd('../')
    end
    
    sizeuCalc = size(uCalc,1);
    if sizeuCalc<size(Numbers.timePoints,2)
        uCalc = [uCalc; zeros(size(Numbers.timePoints,2) - sizeuCalc, ...
            Numbers.compAll(conditionNumber))]; %#ok<AGROW>
    end
    
    u = [u uCalc]; %#ok<AGROW>
    
end


end

