function [y_out,t,u]=...
    PBPK_model(x_in, Model, Numbers, ForODE, ...
    loopNumber, conditionNumber, isCNM) %#ok<INUSL>

%% Preparation
if nargin == 6, isCNM = 1; end
if isCNM
    y_outEq = ForODE.y_outEqAll{conditionNumber};
end

%% Calculation with corresponding functions
[t,u] = toSolveODE(x_in,Numbers,ForODE,loopNumber,conditionNumber);

%% y_out calculation

y_out = [];
if isCNM
    y_out = zeros(1,Numbers.goalEach(conditionNumber));
    [t,y_out] = ...
        toCalcYstar(x_in,Numbers,ForODE,...
        loopNumber,conditionNumber,t,u,y_outEq,y_out); 
end

end




%% toSolveODE
function [t,u] = toSolveODE(x_in,Numbers,ForODE,loopNumber,conditionNumber)

switch char(ForODE.calcAlgorithm(1))
    
    case 'ode_mfile'
        if size(x_in,1) == numel(x_in), x_in = x_in'; end
        
        cd('solveODE')
        
        mfunc = ForODE.mfuncAll{conditionNumber};
        
        for k1 = 1:numel(ForODE.odeSolvers)
            [t,u] = mfunc(ForODE.odeSolvers{k1}, x_in, Numbers.timePoints, ...
                ForODE.jacobian_indexAll{conditionNumber}, ...
                Numbers.compAll(conditionNumber),...
                Numbers.doseSin,Numbers.doseInf);
            if t(end)==Numbers.timePoints(end), break; end
        end
        
        cd('../')
        
    case 'FILT'
        cd('solveFILT')
        [t,u] = solveFILT(x_in, Numbers, ForODE, conditionNumber);
        cd('../')
end


if Numbers.timePoints(2) == 0
   u = [u(1,:) u]; 
end

if sum(sum(u==Inf))~=0&&t(1)~=-8, t = -3; 'u reached Inf',u , end %#ok<NOPRT>

end




%% toCalcYstar
function [t,y_out] = ...
    toCalcYstar(x_in,Numbers,ForODE,loopNumber,conditionNumber,t,u,y_outEq,y_out)

if t(end)==Numbers.timePoints(end) %calc y_out if a simulation completed successfully
    u1 = zeros(size(Numbers.timePoints,2), Numbers.goalEach(conditionNumber));
    if size(u,1)~= size(Numbers.timePoints,2)
        t
        u
        error('Wrong ODE calculation')
    end
    
    for k = 1:Numbers.goalEach(conditionNumber)
        u1(:,k) = u(:,ForODE.y_outCompNumEachCond{conditionNumber}(k));
    end
    %[Numbers.timePoints',u1]
    y_out = calcY_stars(Numbers.timePoints',u1,y_outEq,Numbers);
    
else %y_out = Inf if a simulation did not completed
    for k=1:Numbers.goalEach(conditionNumber)
        y_out(k) = Inf;
    end
end

nan_indexY = find(isnan(y_out)); l_nan=numel(nan_indexY);
if l_nan~=0, t = -4, 'y_out reached NaN', end %#ok<NOPRT>

end

