function y_out = calcY_stars_back(t,u,y_outEq,Numbers)
%Calculate y_star automatically from time-couse
%   Detailed explanation goes here

aZero = Numbers.actTC_addZero;
num_tau = numel(Numbers.timePoints)-aZero;
numbersCompCalc = size(u,2);
y_out = zeros(1,numbersCompCalc);

% Ignore imaginary numbers
u = real(u);

%calc y_out if a simulation completed successfully
if t(end)==Numbers.timePoints(end) 
    cmax=max(u);
    
    kel     = zeros(1,numbersCompCalc);
    AUC     = zeros(1,numbersCompCalc);
    AUCt0   = zeros(1,numbersCompCalc);
    AUCinf  = zeros(1,numbersCompCalc);
    AUCt0inf= zeros(1,numbersCompCalc);
    AUCtau  = zeros(num_tau,numbersCompCalc);
    AUCt0tau= zeros(num_tau,numbersCompCalc);
    
    
    
    for k = 1:numbersCompCalc
        
        %% Prepare t and u
        tu  = [t u(:,k)]; 
        %tu((tu(:,2)==0)&(tu(:,1)~=0),:) = [];
        
        %% Calculate kel
        tendindex = min([size(tu,1), ...
            find(tu(:,2)>(cmax(k)*Numbers.extrapolateAUCthres),...
            1,'last')]);
        tutend = tu(1:tendindex,:);
        
        kel(k)  = kelcalc(tutend,Numbers.extrapolateAUCinf);
        
        %% Calculate AUCs
        AUC(k)  = AUCcalc(tu);
        AUCt0(k)= AUCcalc(tu((1+aZero):end,:));
        
        for ktau = 1:num_tau
            AUCtau(  ktau,k) = AUCcalc(tu(1:(ktau+aZero),:));
        end
        for ktau = 2:num_tau
            AUCt0tau(ktau,k) = AUCcalc(tu((1+aZero):(ktau+aZero),:));
        end
        
        AUCinf(k)  = AUCinfcalc(tutend,                  kel(k));
        AUCt0inf(k)= AUCinfcalc(tutend((1+aZero):end,:), kel(k));
        
    end
    
    for k=1:numbersCompCalc
        %y_outEq(k)
        %[char(y_outEq(k)) num2str(k) ')']
        
        y_out(k) = eval([char(y_outEq(k)) num2str(k) ')']);
    end
    
else %y_out = Inf if a simulation did not completed
    for k=1:numbersCompCalc
        y_out(k) = Inf;
    end
end

end




function kelcalc = kelcalc(tutend,extrapolateAUCinf)

tutend((tutend(:,2)==0)&(tutend(:,1)~=0),:) = [];

t = tutend(:,1); u = tutend(:,2);
tinitindex = size(tutend,1)-extrapolateAUCinf+1;
kelcalc=zeros(2,1);

%If having enough data points and positive u values, calculate kel
%kel otherwise equals 0
if tinitindex > 0 && sum(u(tinitindex:end)<=0)==0
    kelcalc = [t(tinitindex:end),ones(extrapolateAUCinf,1)]...
        \log(u(tinitindex:end));
end
kelcalc = -kelcalc(1);

end




function AUCcalc = AUCcalc(tu)

tu((tu(:,2)==0)&(tu(:,1)~=0),:) = [];

if size(tu,1) == 0
    AUCcalc = 0;
else
    tuplus  = tu; tuplus(1,:)=[];
    tuminus = tu; tuminus(end,:)=[];
    
    tuinterval = tuplus(:,1)-tuminus(:,1);
    uSum = tuplus + tuminus;
    uSum(:,1)=[];
    
    AUCcalc = tuinterval' * uSum / 2;
    
end

end




function AUCinfcalc = AUCinfcalc(tu, kelcalc)

AUC = AUCcalc(tu);
        
if kelcalc<=0
    AUCinfcalc = Inf;
else
    AUCinfcalc = AUC + tu(end,2)./kelcalc;
end

end




function test %#ok<DEFNU>

t       = [0;1;2;3];
u       = [0,0;5,4;3,2;1,1];
y_outEq = {'AUCtau(2,','AUCt0tau(2,'};

Numbers = struct;
Numbers.timePoints    = [0;1;2;3];
Numbers.actTC_addZero = 1;
Numbers.extrapolateAUCthres = 0.001;
Numbers.extrapolateAUCinf   = 3;

y_out = calcY_stars(t,u,y_outEq,Numbers) %#ok<NASGU,NOPRT>

end