function [sumlogdiffBest,FinalXsorted] = ...
    compareTimecourse(f, f2, Model, resultFolder, ...
    XforCalculation, Numbers, ForODE, tcell1, ucell1, Y_k_half, divID) %#ok<INUSL>

'compare time course' %#ok<NOPRT>

XforCalculation = XforCalculation';

%% Prepare data

if nargin==8
    
    tcell1 = cell(Numbers.samp,1);
    ucell1 = cell(Numbers.samp,1);
    Y_k_half=zeros(Numbers.goal,Numbers.samp);
    
    %% solve ODE or FILT
    parfor k1=1:Numbers.samp
        Y_k_halfCalc = []; ufit = [];
        for k2 = 1:Numbers.sepCond %#ok<PFBNS>
            [youtCalc,t,uCalc]=...
                PBPK_model(XforCalculation(k1,:),Model,Numbers,ForODE,1,k2); %#ok<ASGLU>
            
            sizeuCalc = size(uCalc,1);
            if sizeuCalc<size(Numbers.timePoints,2)
                uCalc = [uCalc; ...
                    zeros(size(Numbers.timePoints,2)-sizeuCalc,Numbers.compAll(k2))];
            end
            
            ufit = [ufit uCalc];
            Y_k_halfCalc = [Y_k_halfCalc youtCalc];
            %tocf(6)
        end
        
        Y_k_half(:,k1) = Y_k_halfCalc';
        tcell1{k1}=Numbers.timePoints';
        ucell1{k1}=ufit;
    end
    
end

%% Transform obtained u cells into an array
ufit2=zeros(size(Numbers.timePoints,2),sum(Numbers.compAll),Numbers.samp);
for k2 = 1:Numbers.samp
    calcTimes = size(ucell1{k2},1);
    ufit2(1:calcTimes,:,k2)=ucell1{k2};%1:sum(Numbers.compAll),k2)=ucell1{k2};
end

not_inf_index=find(sum(Y_k_half,1)~=Inf);
calc_index2 = not_inf_index;%calc_index(not_inf_index);

l_posi2=size(calc_index2,2) %#ok<NOPRT>
FinalX=XforCalculation(not_inf_index,:);
FinalY=Y_k_half(:,not_inf_index);
ufit3=ufit2(:,:,not_inf_index);

tcell2=tcell1(not_inf_index);
ucell2=ucell1(not_inf_index);

%% calc differences
sumlogdiff = zeros(l_posi2,1);
for k1=1:l_posi2
    for k2=1:Numbers.timeFitCompNum
        clear u1
        clear u2
        %combine actual and calculated value
        
        u1(:,1:2)=[Numbers.timePoints',Numbers.actTimeCourse(:,k2)];
        u1(:,3)=ufit3(:,Numbers.timeFitComp(k2),k1);
        
        %erase zero
        timeIndex=find(u1(:,2)~=0);
        sizetimeIndex=size(timeIndex,1);
        u2(1:sizetimeIndex,1:2)=u1(timeIndex,2:3);
        
        %calc difference
        sumlogdiff(k1) = sumlogdiff(k1) + sum((log(u2(:,1)./u2(:,2)).^2));
    end
end


%% sort data
[sumlogdiffSorted,sumlogdiffIndex]=sort(sumlogdiff);
sumlogdiffBest = sum(sumlogdiffSorted(1));

FinalXsorted = FinalX(sumlogdiffIndex,:);
FinalYsorted = FinalY(:,sumlogdiffIndex)';
tcell = tcell2(sumlogdiffIndex,:);
ucell = ucell2(sumlogdiffIndex,:);

%% output
csvwrite([resultFolder 'finalXY/FinalXsorted.csv'], FinalXsorted')
csvwrite([resultFolder 'finalXY/FinalYsorted.csv'], FinalYsorted')
csvwrite([resultFolder 'finalXY/sumlogdiffSorted.csv'], sumlogdiffSorted')

%save([resultFolder 'tcell'], 'tcell')
%save([resultFolder 'ucell'], 'ucell')
save([resultFolder 'finalXY/FinalXsorted'], 'FinalXsorted')
save([resultFolder 'finalXY/FinalYsorted'], 'FinalYsorted')
save([resultFolder 'finalXY/sumlogdiffSorted'], 'sumlogdiffSorted')

sumlogdiffIndexForX = not_inf_index(sumlogdiffIndex); %#ok<NASGU>
save([resultFolder 'finalXY/sumlogdiffIndexForX'], 'sumlogdiffIndexForX')

%figure
figure(1)
finalSamples_num = l_posi2;
for k1 = 1:Numbers.timeFitCompNum
    subplot(2,Numbers.timeFitCompNum,k1);
    semilogy(tcell{1}, ucell{1}(:,Numbers.timeFitComp(k1)));
    hold on
    for k2 = 2:finalSamples_num
        plot(tcell{k2}, ucell{k2}(:,Numbers.timeFitComp(k1)));
    end
    plot(Numbers.timePoints, Numbers.actTimeCourse(:,k1), 'mo',...
        'MarkerEdgeColor', [1,0,0], 'MarkerFaceColor', [1,0.5,0],...
        'LineWidth', 1.2, 'MarkerSize', 6);
    hold off
    xlim(Numbers.xlim)
    ylim(Numbers.ylim)
end
finalSamples_num = min(3,l_posi2);
for k1 = 1:Numbers.timeFitCompNum
    subplot(2,Numbers.timeFitCompNum,k1+Numbers.timeFitCompNum);
    semilogy(tcell{1}, ucell{1}(:,Numbers.timeFitComp(k1)));
    hold on
    for k2 = 2:finalSamples_num
        plot(tcell{k2}, ucell{k2}(:,Numbers.timeFitComp(k1)));
    end
    plot(Numbers.timePoints, Numbers.actTimeCourse(:,k1), 'mo',...
        'MarkerEdgeColor', [1,0,0], 'MarkerFaceColor', [1,0.5,0],...
        'LineWidth', 1.2, 'MarkerSize', 6);
    hold off
    xlim(Numbers.xlim)
    ylim(Numbers.ylim)
end

S = get(0, 'ScreenSize');
set(figure(1),'position', [S(3)-750 50 700 700]);
saveas(figure(1),[resultFolder 'result.fig'])
saveas(figure(1),[resultFolder ['../result' num2str(divID) '.png']])

end