function [x_outReturn,AIC,exportSS] = mainLMeach(resultFolder,Setting,parameters)
tic;

Model    = Setting.Model;
calcType = Setting.calcType;

resultFolder2 = resultFolder; 
resultFolStat =[resultFolder2 'stat/'];
mkdir(resultFolStat);

csvwrite([resultFolder2 '/x_in.csv'], parameters')

%% Main loop
% parameters
parDiv = size(parameters,1) - 3;
    
disp(char(10));
display(['#Conditions = ',num2str(parDiv)]);

x_out   = zeros(size(parameters,2),parDiv);
x_stdev = zeros(size(parameters,2),parDiv);
x_outTF   = zeros(size(parameters,2),parDiv);
x_stdevTF = zeros(size(parameters,2),parDiv);
uForFig = cell(1,parDiv);
AIC   = zeros(1,parDiv);
saveSS= zeros(1,parDiv);

if parDiv >1
    matlabpool size; if ans == 0, matlabpool open; end  %#ok<NOANS>
end


parfor kParDiv = 1:parDiv
    
    %% Prepare parameters
    Numbers = ...
        struct('extrapolateAUCthres', 1/1000,'extrapolateAUCinf', 3,'kParDiv',kParDiv);
    IniXY  = struct;
    ForODE = struct; 
    ParaInd= struct;
    
    resultFolder = [resultFolder2 num2str(kParDiv) '/'];
    %mkdir(resultFolder);
    
    cd('../import');
    [ForODE,Numbers,IniXY,ParaInd] = ...
        importParameters(Model,Numbers,IniXY,ForODE,ParaInd,calcType)

    ForODE.odeSolvers = Setting.odeSolvers;  %#ok<PFBNS>
    ForODE.weight     = Setting.weight;
    ForODE.algorithm  = Setting.algorithm;
    cd('../calc');
    
    %save([resultFolder 'Numbers'], 'Numbers')
    
    %% Calculation
    %stopCalc = 0;
    
    [x_outEach, x_stdevEach, x_outTFEach, x_stdevTFEach, t,u,AICEach,SSEach,statOpt] = ...
        LMmethod(resultFolder, IniXY, Numbers, ...
        ForODE, ParaInd, 1, IniXY.xfree, Model); %#ok<ASGLU>
    
    x_out(:,kParDiv)  = x_outEach;
    x_stdev(:,kParDiv)= x_stdevEach;
    x_outTF(:,kParDiv)  = x_outTFEach;
    x_stdevTF(:,kParDiv)= x_stdevTFEach;
    AIC(kParDiv)    = AICEach;
    saveSS(kParDiv) = SSEach;
    
    uForFig{kParDiv} = u;
    
    
    switch statOpt.stat
        case 1
            csvwrite([resultFolStat,num2str(kParDiv),'.csv'], SSEach)
            
            space = repmat(' ',1,max(0,3-numel(num2str(kParDiv))));
            disp(['kParDiv = ',space,num2str(kParDiv),...
                ', SS = ',num2str(SSEach),...
                ', flag = ',num2str(statOpt.flag)])
        case 0
            csvwrite([resultFolStat,num2str(kParDiv),'_failure.csv'], 0)
        case -1
            errStr    = cell(2,1);
            errStr{1} = ['LMM stopped in #',num2str(kParDiv),...
                ' after ',num2str(statOpt.timeLM/60),' min'];
            errStr{2} = ['Iteration = ',num2str(statOpt.iteration),...
                ', Funccount = ',num2str(statOpt.funccount)];
            
            disp(errStr)
            
            csvwrite([resultFolStat,num2str(kParDiv),'_fail_LMtime.csv'], 0)
        case -3
            % levenbergMarquardt at line 313
            % LM parameter too large, cannot proceed or too many function evals
            
            disp(['kParDiv = ',num2str(kParDiv),...
                ', LM parameter too large'])
            
            csvwrite([resultFolStat,num2str(kParDiv),'_fail_LMparam.csv'], 0)
    end
    
end
%end

%% Output
x_outReturn = x_out;
x_outstd = [x_out; zeros(1,parDiv); x_stdev];

%csvwrite([resultFolder2 '/x_out.csv'], x_out)
%csvwrite([resultFolder2 '/AIC.csv'], AIC')
%csvwrite([resultFolder2 '/saveSS.csv'], saveSS')
dlmwrite([resultFolder2 '/x_outstd.csv'], x_outstd, 'precision', 4, 'delimiter', ',')
dlmwrite([resultFolder2 '/AIC.csv'],      AIC',     'precision', 4, 'delimiter', ',')
dlmwrite([resultFolder2 '/saveSS.csv'],   saveSS',  'precision', 4, 'delimiter', ',')

if nargout > 2
    exportSS = saveSS;
end


%x_ss = [1:parDiv; zeros(1,parDiv); saveSS; zeros(1,parDiv); x_out];
[ss_sort,ss_ind] = sort(saveSS);
x_out_sort = x_out(:,ss_ind);
x_std_sort = x_stdev(:,ss_ind);
x_outTF_sort = x_outTF(:,ss_ind);
x_stdTF_sort = x_stdevTF(:,ss_ind);

%x_ss_summary = zeros(4+size(parameters,2),parDiv*2);
x_ss_summary = [];
x_ss_summaryTF = [];
for k = 1:parDiv
    x_ss = [ss_ind(k),ss_ind(k)+0.1;zeros(1,2)];
    x_ss = [x_ss; ss_sort(k),ss_sort(k);zeros(1,2)]; %#ok<AGROW>
    x_ss = [x_ss;x_out_sort(:,k),x_std_sort(:,k)]; %#ok<AGROW>
    x_ss_summary = [x_ss_summary,x_ss]; %#ok<AGROW>
    
    x_ssTF = [ss_ind(k),ss_ind(k)+0.1;zeros(1,2)];
    x_ssTF = [x_ssTF; ss_sort(k),ss_sort(k);zeros(1,2)]; %#ok<AGROW>
    x_ssTF = [x_ssTF;x_outTF_sort(:,k),x_stdTF_sort(:,k)]; %#ok<AGROW>
    x_ss_summaryTF = [x_ss_summaryTF,x_ssTF]; %#ok<AGROW>
end

dlmwrite([resultFolder2 '/x_ss_summary.csv'],   x_ss_summary,   'precision', 4, 'delimiter', ',')
dlmwrite([resultFolder2 '/x_ss_summaryTF.csv'], x_ss_summaryTF, 'precision', 4, 'delimiter', ',')


%% Import Numbers
cd('../import');
Numbers = struct('kParDiv',1);
IniXY  = struct;
ForODE = struct; 
ParaInd= struct;
[~,Numbers,~,~] = ...
    importParameters(Model,Numbers,IniXY,ForODE,ParaInd,calcType);
cd('../calc');


%% Combine u values for figure
% Extract u for figure
uFig = zeros(size(uForFig{1},1),parDiv,Numbers.timeFitCompNum);
for k = 1:parDiv
    u = uForFig{k};
    for k1 = 1:Numbers.timeFitCompNum
        uFig(:,k,k1) = u(:,Numbers.timeFitComp(1,k1));
    end
end

%% Create figure

FigSet.parDiv = parDiv;

FigSet.actTimeCourse = Numbers.actTimeCourse;
FigSet.timeFitComp   = Numbers.timeFitComp;
FigSet.timeFitCompNum= Numbers.timeFitCompNum;
FigSet.timePoints    = Numbers.timePoints;
FigSet.xlim          = Numbers.xlim;
FigSet.ylim          = Numbers.ylim;

FigSet.saveSS = saveSS;
FigSet.x_out  = x_outReturn;

cd('../')
h = createFigLM(FigSet,uFig,2,[],2);
cd('calc')
   
save([resultFolder2 'FigSet'], 'FigSet');
save([resultFolder2 'uFig'],   'uFig');
    
saveas(h,[resultFolder2 'result.fig'])    
saveas(h,[resultFolder2 'result.png'])

%% Close
csvwrite([resultFolder2 'tocf.csv'], tocf(6))
tocf(6)


end