function [t,u] = ...
    sODE_1com_po_ddi(fode,x_in,timePoints,jacobian_index,num_comp,doseSin,doseInf)
%SOLVEODE Solve ode
% Add explanation here

%% Compartments
%  1 Central
%  2 Peripheral
%  3 Intestine
%  4 Intestinel Transit
%

%% Parameters definitions
global x volumeInv startTime odeStopped odeStopTime

warning('off','MATLAB:ode15s:IntegrationTolNotMet')

startTime = tic; 
odeStopped = 0;
odeStopTime= 15; % sec

x=(x_in);


%CALCULATE inverse of compartment volumes
volumeInv = ones(num_comp,1);
volumeInv([1,3]) = 1./x(2);
volumeInv(5)   = 1./x(7);

%% Avoid division with zero
denomiParaInd = [2,7,10];
if sum(x_in(denomiParaInd)==0)
    error(['Divide with zero: #parameter(s) = ',...
        num2str(denomiParaInd(x_in(denomiParaInd)==0))])
end


%% Main calculation
options=odeset('Events',@events,'JPattern', jacobian_index);
[t,u] = calc_SepTP(fode, options, num_comp,timePoints,doseSin,doseInf);

end




%% Prepare fluxmatrix w/o u
function flux = calcFluxPre(x,t) %#ok<INUSD>

%% Substrates

fluxSubs = zeros(2);

fluxSubs(1,1) =-x(1);      % Central to Elimination
fluxSubs(1,2) = x(3);      % Intes to Central


%% Inhibitors

fluxInhi = zeros(1);

fluxInhi(1,1) =-x(6);      % Central to Elimination


%% Combine flux

z = zeros(2);

flux = [...
    fluxSubs, z;...
    z, fluxSubs;...
    ];

flux = [...
    flux,    zeros(4,1);...
    zeros(1,4),fluxInhi;...
    ];


end



%% Prepare fluxmatrix with u
function flux = calcFluxMid(x,t,u,fluxPre) %#ok<INUSL>


%% Combine and Put -(sum(outflow)) at flux(k,k) to keep mass balance

flux = fluxPre;

flux(3,3) = flux(3,3)/(1 + u(5)/x(10));

fluxDiag=sum(flux - diag(diag(flux)),1);
flux = flux - diag(fluxDiag);


end






%% %%%%%%%%%%%%% Do not modify below  %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [t,u] = calc_SepTP(fode, options, num_comp, timePoints,doseSin,doseInf)
global x doseInfusion_calcEach fluxPre odeStopped %kpInv

%% Prepare flux w/o cells containing u
fluxPre = calcFluxPre(x);

uinitial = zeros(1,num_comp);

% Separate time points
[tIni,tIniEraseIndex,timePointsSep,doseSingle_calc,doseInfusion_calc] = ...
    sepTimePoints(doseSin, doseInf, x, num_comp, timePoints);

% Initialize
tcombine = []; ucombine = [];

% Calculate ODE for each separated range
for k = 1: size(timePointsSep,2)

    % Initial setting for each cycle
    tp = timePointsSep{k};
    twoTimePoints = 0;
    
    if k>1, uinitial = uend; end
    
    uinitial = uinitial + doseSingle_calc(k,:);% Add infusion rate
    
    % If contain only two time points
    if size(timePointsSep{k},2) == 2
        twoTimePoints = 1;
        tp = [tp(1) (tp(1)+tp(2))/2 tp(2)];
    end
    
    % Decide calculate status
    calcStat = (sum(abs(uinitial))>0) + 2*(sum(abs(doseInfusion_calc(k,:)))>0);

    switch calcStat
        case 0 % No drug administered. Return zero
            t = tp';
            u = zeros(size(tp,2), num_comp);
        case 1 % W/O infusion
            [t,u]=fode(@RHS, tp, uinitial', options);
        case {2,3} % With infusion
            doseInfusion_calcEach = doseInfusion_calc(k,:);
            [t,u]=fode(@RHS_infusion, tp, uinitial', options);
    end
    if t(end) ~= tp(end), odeStopped = 1; break; end
    
    if twoTimePoints, t(2) = []; u(2,:) = []; end
    
    % Erase end point except for final loop
    if k <size(timePointsSep,2)
        uend =  u(end,:);
        t(end) = []; u(end,:) = [];
    end
    t = t + tIni(k);
    
    if tIniEraseIndex(k)
        t(1) = []; u(1,:) = [];
    end
    
    
    tcombine = [tcombine;t]; %#ok<AGROW> 
    ucombine = [ucombine;u]; %#ok<AGROW>
end

if odeStopped
    t = [-8,t(end)];
    u = ones(1, num_comp) * Inf;
else
    t = tcombine;
    u = ucombine;
end



end

%% Interface of ODE function: Normal administration
function [hReturn]=RHS(t,u)
global fluxPre volumeInv x startTime odeStopped odeStopTime

%['RHS0: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u; % Calculate differentials
hReturn = hReturn .* volumeInv; % Divide with volumes
%save('hReturn')

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end



%% Interface of ODE function: Complicated administration
function [hReturn]=RHS_infusion(t,u)
global fluxPre volumeInv x doseInfusion_calcEach startTime odeStopped odeStopTime

%['RHS1: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u;% Calculate differentials
hReturn = hReturn .* volumeInv;% Divide with volumes 
hReturn = hReturn + doseInfusion_calcEach';% Add infusion rate
%hReturn'

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end




%%
function [value,isterminal,direction] = events(t,u) %#ok<INUSD>
global odeStopped

value = odeStopped - 1;
isterminal = 1;
direction = 0;

end