function [t,u] = ...
    sODE_MDZ_TPV_ivpo(fode,x_in,timePoints,jacobian_index,num_comp,doseSin,doseInf)
%SOLVEODE Solve ode
% Add explanation here

%% Compartments
%  1-35: control
% 36-70: DDI

%% Parameters definitions
global x volumeInv startTime odeStopped odeStopTime

warning('off','MATLAB:ode15s:IntegrationTolNotMet')

startTime = tic; 
odeStopped = 0;
odeStopTime= 15; % sec

x=(x_in);


%CALCULATE inverse of compartment volumes
volumeInv = ones(num_comp,1);
volumeInv(1:10:31) = 1./x_in(11);
volumeInv(41)      = 1./x_in(21);

volumeInv( 3: 7) = 1./(x_in(2)+x_in(3));
volumeInv(13:17) = 1./(x_in(2)+x_in(3));
volumeInv(23:27) = 1./(x_in(2)+x_in(3));
volumeInv(33:37) = 1./(x_in(2)+x_in(3));
volumeInv(43:47) = 1./(x_in(2)+x_in(3));


%% Avoid division with zero
denomiParaInd = [2,3,6,7,8,11,21,13,14,23,24];
if sum(x_in(denomiParaInd)==0)
    error(['Divide with zero: #parameter(s) = ',...
        num2str(denomiParaInd(x_in(denomiParaInd)==0))])
end


%% Main calculation
options=odeset('Events',@events,'JPattern', jacobian_index);
[t,u] = calc_SepTP(fode, options, num_comp,timePoints,doseSin,doseInf);

end




%% Prepare fluxmatrix w/o u
function flux = calcFluxPre(x)

%% Drugs
fluxMDZ = createSmallflux(x(11:20),x(1));
fluxTPV = createSmallflux(x(21:30),x(1));

%% Combine flux
z = zeros(10,10);

flux = [...
    fluxMDZ,z,z,z,z;...
    z,fluxMDZ,z,z,z;...
    z,z,fluxMDZ,z,z;...
    z,z,z,fluxMDZ,z;...
    z,z,z,z,fluxTPV;...
    ];

flux = [flux,zeros(50,5); zeros(5,50),zeros(5,5)];

end

function flux = createSmallflux(x,qh)

flux = zeros(10);

%% Rapid and peripheral
flux(3,1) = qh;            % rapid to Liver 1
flux(1,1) =-x(7);          % Rapid to Urine

flux(2,1) = x(5);          % rapid to peripheral
flux(1,2) = x(6);          % peripheral to rapid

%% Intestine
flux(3,8) = x(2)*x(3);     % intestine to HE1
flux(8,8) =-x(2)*(1-x(3)); % intestine to feces

%% Metabolism
fBCLint_5_Kp  = x(8) * x(9) / x(4) * 0.2;
flux(3:7,3:7) = flux(3:7,3:7) - fBCLint_5_Kp * eye(5); 

%% Hepatic Blood
flux(4,3) = qh/x(4);
flux(5,4) = qh/x(4);
flux(6,5) = qh/x(4); 
flux(7,6) = qh/x(4);
flux(1,7) = qh/x(4); % Liver 5 to rapid

end



%% Prepare fluxmatrix with u
function flux = calcFluxMid(x,t,u,flux) %#ok<INUSL>

%% CL change
fm3A4 = 1 - 1/(1+x(20));
r_3A4 = u(51:55)./(1+(x(28)./x(24).*u(43:47)./x(31)));
r_CL  = (1-fm3A4) + fm3A4 * r_3A4;

flux(23,23) = flux(23,23) * r_CL(1);
flux(24,24) = flux(24,24) * r_CL(2);
flux(25,25) = flux(25,25) * r_CL(3);
flux(26,26) = flux(26,26) * r_CL(4);
flux(27,27) = flux(27,27) * r_CL(5);

flux(33,33) = flux(33,33) * r_CL(1);
flux(34,34) = flux(34,34) * r_CL(2);
flux(35,35) = flux(35,35) * r_CL(3);
flux(36,36) = flux(36,36) * r_CL(4);
flux(37,37) = flux(37,37) * r_CL(5);

%% MBI

cu =  x(28)./x(24).*u(51:55);

flux(51,51) = x(32)*(1/u(51)-1) - x(34) * cu(1) / (cu(1) + x(33));
flux(52,52) = x(32)*(1/u(52)-1) - x(34) * cu(2) / (cu(2) + x(33));
flux(53,53) = x(32)*(1/u(53)-1) - x(34) * cu(3) / (cu(3) + x(33));
flux(54,54) = x(32)*(1/u(54)-1) - x(34) * cu(4) / (cu(4) + x(33));
flux(55,55) = x(32)*(1/u(55)-1) - x(34) * cu(5) / (cu(5) + x(33));



%% Combine and Put -(sum(outflow)) at flux(k,k) to keep mass balance

fluxDiag=sum(flux - diag(diag(flux)),1);
flux = flux - diag(fluxDiag);


end






%% %%%%%%%%%%%%% Do not modify below  %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [t,u] = calc_SepTP(fode, options, num_comp, timePoints,doseSin,doseInf)
global x doseInfusion_calcEach fluxPre odeStopped %kpInv

%% Prepare flux w/o cells containing u
fluxPre = calcFluxPre(x);

uinitial = zeros(1,num_comp);

% Separate time points
[tIni,tIniEraseIndex,timePointsSep,doseSingle_calc,doseInfusion_calc] = ...
    sepTimePoints(doseSin, doseInf, x, num_comp, timePoints);

% Initialize
tcombine = []; ucombine = [];

% Calculate ODE for each separated range
for k = 1: size(timePointsSep,2)

    % Initial setting for each cycle
    tp = timePointsSep{k};
    twoTimePoints = 0;
    
    if k>1, uinitial = uend; end
    
    uinitial = uinitial + doseSingle_calc(k,:);% Add infusion rate
    
    % If contain only two time points
    if size(timePointsSep{k},2) == 2
        twoTimePoints = 1;
        tp = [tp(1) (tp(1)+tp(2))/2 tp(2)];
    end
    
    % Decide calculate status
    calcStat = (sum(abs(uinitial))>0) + 2*(sum(abs(doseInfusion_calc(k,:)))>0);

    switch calcStat
        case 0 % No drug administered. Return zero
            t = tp';
            u = zeros(size(tp,2), num_comp);
        case 1 % W/O infusion
            [t,u]=fode(@RHS, tp, uinitial', options);
        case {2,3} % With infusion
            doseInfusion_calcEach = doseInfusion_calc(k,:);
            [t,u]=fode(@RHS_infusion, tp, uinitial', options);
    end
    if t(end) ~= tp(end), odeStopped = 1; break; end
    
    if twoTimePoints, t(2) = []; u(2,:) = []; end
    
    % Erase end point except for final loop
    if k <size(timePointsSep,2)
        uend =  u(end,:);
        t(end) = []; u(end,:) = [];
    end
    t = t + tIni(k);
    
    if tIniEraseIndex(k)
        t(1) = []; u(1,:) = [];
    end
    
    
    tcombine = [tcombine;t]; %#ok<AGROW> 
    ucombine = [ucombine;u]; %#ok<AGROW>
end

if odeStopped
    t = [-8,t(end)];
    u = ones(1, num_comp) * Inf;
else
    t = tcombine;
    u = ucombine;
end



end

%% Interface of ODE function: Normal administration
function [hReturn]=RHS(t,u)
global fluxPre volumeInv x startTime odeStopped odeStopTime

%['RHS0: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u; % Calculate differentials
hReturn = hReturn .* volumeInv; % Divide with volumes
%save('hReturn')

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end



%% Interface of ODE function: Complicated administration
function [hReturn]=RHS_infusion(t,u)
global fluxPre volumeInv x doseInfusion_calcEach startTime odeStopped odeStopTime

%['RHS1: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u;% Calculate differentials
hReturn = hReturn .* volumeInv;% Divide with volumes 
hReturn = hReturn + doseInfusion_calcEach';% Add infusion rate
%hReturn'

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end




%%
function [value,isterminal,direction] = events(t,u) %#ok<INUSD>
global odeStopped

value = odeStopped - 1;
isterminal = 1;
direction = 0;

end