function [t,u] = ...
    sODE_didb_multi(fode,x_in,timePoints,jacobian_index,num_comp,doseSin,doseInf) %#ok<INUSL>
%SOLVEODE Solve ode
% Add explanation here

% x_in
% jacobian_index

%% Compartments
%% 1-10: Parent 
%  1 Rapid
%  2 Intestine
%  3 Liver
%  4 Peripheral
%  5 Urine
%  6 Intestinel Transit
%
%% 11-40: Metabolites 1-3 
%  1 Rapid
%  2 Intestine
%  3 Liver
%  4 Peripheral
%  5 Urine
%
%% 41-46,51-56: Inhibitor 1,2
%  1 Rapid
%  2 Intestine
%  3 Liver
%  4 Peripheral
%  5 Urine
%  6 Intestinel Transit

%% Parameters definitions
global x volumeInv startTime odeStopped odeStopTime

warning('off','MATLAB:ode15s:IntegrationTolNotMet')

startTime = tic; 
odeStopped = 0;
odeStopTime= 15; % sec

x_in = tfCLint(x_in);

%CALCULATE inverse of compartment volumes
volumeInv = ones(60,1);
volumeInv( 1:10:51) = 1./x_in([4,41,51,61,71,81]); % Rapid
volumeInv( 3:10:53) = 1./x_in(2);            % Liver

num_comp = 60;

%% Avoid division with zero
denomiParaInd = [2,4,7,41,51,61,43,53,63,71,74,75,81,84,85];
if sum(x_in(denomiParaInd)==0)
    error(['Divide with zero: #parameter(s) = ',...
        num2str(denomiParaInd(x_in(denomiParaInd)==0))])
end


%% Main calculation
options=odeset('Events',@events,'JPattern', jacobian_index);
uAll = [];

for cond = 1:4
    x = changeInhibition(x_in,cond);
    
    [t,u] = calc_SepTP(fode, options, num_comp, timePoints,doseSin,doseInf);
    
    % Correct MW
    u(:,11:20) = u(:,11:20) * x(91);
    u(:,21:30) = u(:,21:30) * x(92);
    u(:,31:40) = u(:,31:40) * x(93);
    
    if size(t,1) ~= numel(timePoints) || size(u,1) ~= numel(timePoints)
        if t(end) == timePoints(end)
            cond
            x
            t
            u
            
            t(end) = 0;
        end
        t = [-8,t(end)];
        uAll = ones(1, 240) * Inf;
        break;
    else
        uAll = [uAll,u]; %#ok<AGROW>
    end
end

u = uAll;

end



function x = changeInhibition(x,cond)

switch cond
    case 1 % Control
        x([80,90]) = 0;
        x(96:109) = 0;
        
    case 2 % Inhibitor 1
        x(90) = 0;
        x(103:109) = 0;
        
    case 3 % Inhibitor 2
        x(80) = 0;
        x(96:102) = 0;
        
    case 4 % Combi
        
end

end



function x = tfCLint(x)

%Change CLint,total and ratios into each CLint

clCYP = x(31) * [x(28:30),1] / sum([x(28:30),1]);

x(16:4:28) = clCYP(1) * [x(16:4:24),1] / sum([x(16:4:24),1]);
x(17:4:29) = clCYP(2) * [x(17:4:25),1] / sum([x(17:4:25),1]);
x(18:4:30) = clCYP(3) * [x(18:4:26),1] / sum([x(18:4:26),1]);
x(19:4:31) = clCYP(4) * [x(19:4:27),1] / sum([x(19:4:27),1]);

x(47:50) = x(50) * [x(47:49),1] / sum([x(47:49),1]);
x(57:60) = x(60) * [x(57:59),1] / sum([x(57:59),1]);
x(67:70) = x(70) * [x(67:69),1] / sum([x(67:69),1]);

end




%% Prepare fluxmatrix w/o u
function flux = calcFluxPre(x,t) %#ok<INUSD>

%% Parent
flux = zeros(10);

flux(3,1) = x(1);      % Rapid to Liver
flux(1,3) = x(1)/x(8); % Liver to Rapid
flux(4,1) = x(9);      % Rapid to Peripheral
flux(1,4) = x(10);     % Peripheral to Rapid
flux(5,1) = x(11);     % Rapid to Urine

kIntesElim = x(5) * (1-x(7))/ x(7);

flux(2,6) = x(6);      % Intes Transit to Intes
flux(3,2) = x(5);      % Intes to Liver
flux(2,2) =-kIntesElim / (1+sum(x(12:14))); % Intes to Feces

fluxPare = flux;



%% Metabolites

fluxMet1 = metaboliteFlux(x(41:50),x(1));
fluxMet2 = metaboliteFlux(x(51:60),x(1));
fluxMet3 = metaboliteFlux(x(61:70),x(1));



%% Inhibitors

fluxInhi1 = inhibitorFlux(x(71:80),x(1));
fluxInhi2 = inhibitorFlux(x(81:90),x(1));


%% Combine flux

z = zeros(10);

flux = [...
    fluxPare, z,z,z,z,z;...
    z,fluxMet1, z,z,z,z;...
    z,z,fluxMet2, z,z,z;...
    z,z,z,fluxMet3, z,z;...
    z,z,z,z,fluxInhi1,z;...
    z,z,z,z,z,fluxInhi2;...
    ];

% Add intestinal metabolism
kIntesArr    = x(12:15) * kIntesElim / (1+sum(x(12:15)));
kIntesArr(4) = -kIntesArr(4);
kIntesArr    = kIntesArr  /(1+x(102)+x(109));
flux([12,22,32,2],2) = kIntesArr;% Intes to Met 1-3 Intes


end




function flux = metaboliteFlux(x,qh)

flux = zeros(10);


flux(3,2) = x(2);     % Intes to Liver

flux(3,1) = qh;       % Rapid to Liver
flux(4,1) = x(4);     % Rapid to Peripheral
flux(5,1) = x(6);     % Rapid to Urine

flux(1,3) = x(1)/x(3);% Liver to Rapid
flux(1,4) = x(5);     % Peripheral to Rapid

end




function flux = inhibitorFlux(x,qh)

flux = zeros(10);

flux(3,1) = qh;       % Rapid to Liver
flux(1,3) = qh/x(5);  % Liver to Rapid
flux(3,3) =-x(9)/x(5);% Liver to elimination
flux(4,1) = x(6);     % Rapid to Peripheral
flux(1,4) = x(7);     % Peripheral to Rapid
flux(5,1) = x(8);     % Rapid to Urine

flux(2,6) = x(3);     % Intes Transit to Intes
flux(3,2) = x(2);     % Intes to Liver
flux(2,2) =-x(2) * (1-x(4))/ x(4) ;% Intes to Feces

end




%% Prepare fluxmatrix with u
function flux = calcFluxMid(x,t,u,fluxPre) %#ok<INUSL>

flux = zeros(60);

%% Add metabolisms
x    = calcCLchange(x,u);
flux = addMetabolism(flux,x);

%% Combine and Put -(sum(outflow)) at flux(k,k) to keep mass balance

flux = flux + fluxPre;

fluxDiag=sum(flux - diag(diag(flux)),1);
flux = flux - diag(fluxDiag);


end




function flux = addMetabolism(flux,x)

flux(13,3) = sum(x(16:19))/x(8); % Liver to Met 1 Liver
flux(23,3) = sum(x(20:23))/x(8); % Liver to Met 2 Liver
flux(33,3) = sum(x(24:27))/x(8); % Liver to Met 3 Liver
flux( 3,3) =-sum(x(28:31))/x(8); % Liver to Met other

flux(13,13)=-sum(x(47:50))/x(43);% Met 1 Hepatic elimination
flux(23,23)=-sum(x(57:60))/x(53);% Met 2 Hepatic elimination
flux(33,33)=-sum(x(67:70))/x(63);% Met 3 Hepatic elimination

end




function x = calcCLchange(x,u)

% Competitive
r_cmp = ones(1,3);
if x(96) > 0, r_cmp(1) = r_cmp(1) + u(43)/x(75)/x(96); end
if x(97) > 0, r_cmp(2) = r_cmp(2) + u(43)/x(75)/x(97); end
if x(98) > 0, r_cmp(3) = r_cmp(3) + u(43)/x(75)/x(98); end
if x(103)> 0, r_cmp(1) = r_cmp(1) + u(53)/x(85)/x(103); end
if x(104)> 0, r_cmp(2) = r_cmp(2) + u(53)/x(85)/x(104); end
if x(105)> 0, r_cmp(3) = r_cmp(3) + u(53)/x(85)/x(105); end

% MBI
r_mbi = (1 + x(99:101)) .* (1 + x(106:108));

% Enzyme 1
x(16:4:28)  = x(16:4:28) /r_cmp(1)/r_mbi(1);
x(47:10:67) = x(47:10:67)/r_cmp(1)/r_mbi(1);

% Enzyme 2
x(17:4:29)  = x(17:4:29) /r_cmp(2)/r_mbi(2);
x(48:10:68) = x(48:10:68)/r_cmp(2)/r_mbi(2);

% Enzyme 3
x(18:4:30)  = x(18:4:30) /r_cmp(3)/r_mbi(3);
x(49:10:69) = x(49:10:69)/r_cmp(3)/r_mbi(3);


end





%% %%%%%%%%%%%%% Do not modify below  %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [t,u] = calc_SepTP(fode, options, num_comp, timePoints,doseSin,doseInf)
global x doseInfusion_calcEach fluxPre odeStopped %kpInv

%% Prepare flux w/o cells containing u
fluxPre = calcFluxPre(x);

uinitial = zeros(1,num_comp);

% Separate time points
[tIni,tIniEraseIndex,timePointsSep,doseSingle_calc,doseInfusion_calc] = ...
    sepTimePoints(doseSin, doseInf, x, num_comp, timePoints);

% Initialize
tcombine = []; ucombine = [];

% Calculate ODE for each separated range
for k = 1: size(timePointsSep,2)

    % Initial setting for each cycle
    tp = timePointsSep{k};
    twoTimePoints = 0;
    
    if k>1, uinitial = uend; end
    
    uinitial = uinitial + doseSingle_calc(k,:);% Add infusion rate
    
    % If contain only two time points
    if size(timePointsSep{k},2) == 2
        twoTimePoints = 1;
        tp = [tp(1) (tp(1)+tp(2))/2 tp(2)];
    end
    
    % Decide calculate status
    calcStat = (sum(abs(uinitial))>0) + 2*(sum(abs(doseInfusion_calc(k,:)))>0);

    switch calcStat
        case 0 % No drug administered. Return zero
            t = tp';
            u = zeros(size(tp,2), num_comp);
        case 1 % W/O infusion
            [t,u]=fode(@RHS, tp, uinitial', options);
        case {2,3} % With infusion
            doseInfusion_calcEach = doseInfusion_calc(k,:);
            [t,u]=fode(@RHS_infusion, tp, uinitial', options);
    end
    if t(end) ~= tp(end), odeStopped = 1; break; end
    
    if twoTimePoints, t(2) = []; u(2,:) = []; end
    
    % Erase end point except for final loop
    if k <size(timePointsSep,2)
        uend =  u(end,:);
        t(end) = []; u(end,:) = [];
    end
    t = t + tIni(k);
    
    if tIniEraseIndex(k)
        t(1) = []; u(1,:) = [];
    end
    
    
    tcombine = [tcombine;t]; %#ok<AGROW> 
    ucombine = [ucombine;u]; %#ok<AGROW>
end

if odeStopped
    t = [-8,t(end)];
    u = ones(1, num_comp) * Inf;
else
    t = tcombine;
    u = ucombine;
end



end

%% Interface of ODE function: Normal administration
function [hReturn]=RHS(t,u)
global fluxPre volumeInv x startTime odeStopped odeStopTime

%['RHS0: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u; % Calculate differentials
hReturn = hReturn .* volumeInv; % Divide with volumes
%save('hReturn')

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end



%% Interface of ODE function: Complicated administration
function [hReturn]=RHS_infusion(t,u)
global fluxPre volumeInv x doseInfusion_calcEach startTime odeStopped odeStopTime

%['RHS1: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u;% Calculate differentials
hReturn = hReturn .* volumeInv;% Divide with volumes 
hReturn = hReturn + doseInfusion_calcEach';% Add infusion rate
%hReturn'

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end




%%
function [value,isterminal,direction] = events(t,u) %#ok<INUSD>
global odeStopped

value = odeStopped - 1;
isterminal = 1;
direction = 0;

end