function [t u] = ...
    sODE_irino_transit(fode,x_in,timePoints,jacobian_index,num_comp,doseSin,doseInf) %#ok<INUSL>
%SOLVEODE Solve ode
% Add explanation here

%% Metabolites
% CPT11 SN38 SN38G NPC APC

%% Compartments
%  1: 5 Rapid
%  6:10 Late
% 11:15 Liver 
% 16:20 Intestine
% 21:25 Urine
% 26:30 Feces -> Combined to [26,27+28,29,30]
% 31:35 Bile
% 36:40 Transit 1
% 41:45 Transit 2
% 46:50 Transit 3
% 51:55 Transit to feces


%% Parameters definitions
global x volumeInv startTime odeStopped odeStopTime

warning('off','MATLAB:ode15s:IntegrationTolNotMet')

startTime = tic; 
odeStopped = 0;
odeStopTime= 60; % sec

x=(x_in);
num_comp = 55; % Before combining metabolites, usually not required

%CALCULATE CLbile,transit & CLbile,Ttube
x(11:15) = x_in(11:15) * 1       *(1+x_in(63));
x(16:20) = x_in(11:15) * x_in(63)*(1+x_in(63));

%CALCULATE inverse of compartment volumes
volumeInv = ones(num_comp,1);
volumeInv(1:5)   = 1 ./ x(56:60); % Rapid compartment
volumeInv(11:15) = 1 ./ x(62);    % Liver compartment

%% Avoid division with zero
denomiParaInd = [1:5,56:60,62];
if sum(x_in(denomiParaInd)==0)
    error(['Divide with zero: #parameter(s) = ',...
        num2str(denomiParaInd(x_in(denomiParaInd)==0))])
end


%% Main calculation
options=odeset('Events',@events,'JPattern', jacobian_index);
[t,u] = calc_SepTP(fode, options, num_comp,timePoints,doseSin,doseInf);



%% Combine time-profiles to export

% For fitting with time points
uTimeP = [sum(u(:,21:25),2),sum(u(:,26:30),2),sum(u(:,31:35),2)];

% Combine SN38 and SN38G in feces
u = [u(:,1:26), u(:,27)+u(:,28), u(:,29:end), uTimeP];

end







%% Prepare fluxmatrix w/o u
function flux = calcFluxPre(x,t) %#ok<INUSD>


% Calculate inverse of Kp
kpInv = ones(1,5)./x(1:5); % size = [5 1]

% Construct small flux matrix
flux1  = diag(x(46:50));       % rapid to late
flux2  = diag(x(51:55));       % late to rapid 
flux3  = eye(5) * x(61);       % rapid to liver 
flux4  = diag(x(61) * kpInv);  % liver to rapid 
flux5  = diag(x(6:10));        % rapid to urine 
flux6  = diag(x(11:15).*kpInv);% liver to transit 1 
flux7  = diag(x(16:20).*kpInv);% liver to bile T-tube
flux8  = diag(x(31:35));       % intestine to liver
flux9  = diag(x(36:40));       % intestine to large intestine
flux11 = diag(x(41:45));       % transit 1 to 2, 2 to 3, 3 to intestine
flux12 = diag(x(26:30));       % large intestine to feces

flux10 = zeros(5);
flux10(2,1) = x(21) * kpInv(1); % CPT11 to SN38 , CES_1
flux10(4,1) = x(22) * kpInv(1); % NPC   to SN38 , CES_2
flux10(5,1) = x(23) * kpInv(1); % CPT11 to APC  , 3A4_1 
flux10(2,4) = x(24) * kpInv(4); % CPT11 to NPC  , 3A4_2
flux10(3,2) = x(25) * kpInv(2); % SN38  to SN38G, UGT 

% Construct full flux matrix
flux = zeros(50);
flux(6:10,  1:5  ) = flux1;
flux(1:5,   6:10 ) = flux2;
flux(11:15, 1:5  ) = flux3;
flux(1:5,   11:15) = flux4;
flux(21:25, 1:5  ) = flux5;
flux(36:40, 11:15) = flux6;
flux(31:35, 11:15) = flux7;
flux(11:15, 16:20) = flux8;
flux(51:55, 16:20) = flux9;
flux(11:15, 11:15) = flux10;
flux(41:45, 36:40) = flux11;
flux(46:50, 41:45) = flux11;
flux(16:20, 46:50) = flux11;
flux(26:30, 51:55) = flux12;


end



%% Prepare fluxmatrix with u
function flux = calcFluxMid(x,t,u,fluxPre) %#ok<INUSL>


%% Combine and Put -(sum(outflow)) at flux(k,k) to keep mass balance

flux = fluxPre;

fluxDiag=sum(flux - diag(diag(flux)),1);
flux = flux - diag(fluxDiag);


end







%% %%%%%%%%%%%%% Do not modify below  %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [t,u] = calc_SepTP(fode, options, num_comp, timePoints,doseSin,doseInf)
global x doseInfusion_calcEach fluxPre odeStopped %kpInv

%% Prepare flux w/o cells containing u
fluxPre = calcFluxPre(x);

uinitial = zeros(1,num_comp);

% Separate time points
[tIni,tIniEraseIndex,timePointsSep,doseSingle_calc,doseInfusion_calc] = ...
    sepTimePoints(doseSin, doseInf, x, num_comp, timePoints);

% Initialize
tcombine = []; ucombine = [];

% Calculate ODE for each separated range
for k = 1: size(timePointsSep,2)

    % Initial setting for each cycle
    tp = timePointsSep{k};
    twoTimePoints = 0;
    
    if k>1, uinitial = uend; end
    
    uinitial = uinitial + doseSingle_calc(k,:);% Add infusion rate
    
    % If contain only two time points
    if size(timePointsSep{k},2) == 2
        twoTimePoints = 1;
        tp = [tp(1) (tp(1)+tp(2))/2 tp(2)];
    end
    
    % Decide calculate status
    calcStat = (sum(abs(uinitial))>0) + 2*(sum(abs(doseInfusion_calc(k,:)))>0);

    switch calcStat
        case 0 % No drug administered. Return zero
            t = tp';
            u = zeros(size(tp,2), num_comp);
        case 1 % W/O infusion
            [t,u]=fode(@RHS, tp, uinitial', options);
        case {2,3} % With infusion
            doseInfusion_calcEach = doseInfusion_calc(k,:);
            [t,u]=fode(@RHS_infusion, tp, uinitial', options);
    end
    if t(end) ~= tp(end), odeStopped = 1; break; end
    
    if twoTimePoints, t(2) = []; u(2,:) = []; end
    
    % Erase end point except for final loop
    if k <size(timePointsSep,2)
        uend =  u(end,:);
        t(end) = []; u(end,:) = [];
    end
    t = t + tIni(k);
    
    if tIniEraseIndex(k)
        t(1) = []; u(1,:) = [];
    end
    
    
    tcombine = [tcombine;t]; %#ok<AGROW> 
    ucombine = [ucombine;u]; %#ok<AGROW>
end

if odeStopped
    t = [-8,t(end)];
    u = ones(1, num_comp) * Inf;
else
    t = tcombine;
    u = ucombine;
end



end

%% Interface of ODE function: Normal administration
function [hReturn]=RHS(t,u)
global fluxPre volumeInv x startTime odeStopped odeStopTime

%['RHS0: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u; % Calculate differentials
hReturn = hReturn .* volumeInv; % Divide with volumes
%save('hReturn')

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end



%% Interface of ODE function: Complicated administration
function [hReturn]=RHS_infusion(t,u)
global fluxPre volumeInv x doseInfusion_calcEach startTime odeStopped odeStopTime

%['RHS1: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u;% Calculate differentials
hReturn = hReturn .* volumeInv;% Divide with volumes 
hReturn = hReturn + doseInfusion_calcEach';% Add infusion rate
%hReturn'

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end




%%
function [value,isterminal,direction] = events(t,u) %#ok<INUSD>
global odeStopped

value = odeStopped - 1;
isterminal = 1;
direction = 0;

end