function [t,u] = ...
    sODE_tpddi_simple_CsA(fode,x_in,timePoints,jacobian_index,num_comp,doseSin,doseInf)
%SOLVEODE Solve ode
% Add explanation here

% x_in
% jacobian_index

%% Compartments
%  Subs
%  1    Rapid
%  2:11 HE1, HC1, ..., HE5, HC5
% 12    Intestine
% 13    Intestinal Transit
% 14:16 Transit 1:3
%
%  Inhi
%  1    Rapid
%  2:2:10 Liver1, ..., Liver5
% 12    Intestine
% 13    Intestinal Transit

%% Parameters definitions
global x volumeInv startTime odeStopped odeStopTime

warning('off','MATLAB:ode15s:IntegrationTolNotMet')

startTime = tic; 
odeStopped = 0;
odeStopTime= 15; % sec

x=(x_in);


%CALCULATE inverse of compartment volumes
volumeInv = ones(num_comp,1);

% Substrate - cont
volumeInv(1)      = 1/x(11); % Rapid
volumeInv(2:2:10) = 5/x(2);  % HE
volumeInv(3:2:11) = 5/x(3);  % HC

% Substrate - DDI
volumeInv(21:40) = volumeInv(1:20); 

% Inhibitor
volumeInv(41)     = 1/x(31); % Rapid
volumeInv(42:2:50)= 5/(x(2)+x(3));  % HE


%% Main calculation
options=odeset('Events',@events,'JPattern', jacobian_index);

[t,u] = calc_SepTP(fode, options, num_comp, timePoints,doseSin,doseInf);

end




%% Prepare fluxmatrix w/o u
function flux = calcFluxPre(x,t) %#ok<INUSD>


%% Prepare flux for substrate
flux = zeros(20);

flux(2,1)  = x(1);   % rapid to HE1
flux(1,1)  =-x(17);  % rapid to urine

flux(17,1) = x(18);  % rapid to distribution
flux(1,17) = x(19);  % distribution to rapid

flux(12,13) = x(13); % Intestinal transit to intestine
flux(2,12)  = x(12); % intestine to HE1
flux(12,12) =-x(21); % intestine to feces

flux(15,14) = x(20); % transit 1 to 2
flux(16,15) = x(20); % transit 2 to 3
flux(12,16) = x(20); % transit 3 to intestine

fBPSinf  = x(14) * 0.2;
fHPSeff  = x(15) * 0.2;
fHCLint  = x(16) * 0.2;
fHCLbile = fHCLint      /(1+x(22));
fHCLmet  = fHCLint*x(22)/(1+x(22));
fluxHEHC = [0, fHPSeff; fBPSinf, -fHCLmet];

flux(2:3,  2 :3 ) = fluxHEHC; 
flux(4:5,  4 :5 ) = fluxHEHC; 
flux(6:7,  6 :7 ) = fluxHEHC; 
flux(8:9,  8 :9 ) = fluxHEHC; 
flux(10:11,10:11) = fluxHEHC; 
flux(4 ,2 ) = x(1);
flux(6 ,4 ) = x(1);
flux(8 ,6 ) = x(1); 
flux(10,8 ) = x(1);
flux(1,10 ) = x(1); % HE5 to rapid
flux(14,[3,5,7,9,11]) = fHCLbile; 


fluxCont = flux;
fluxDDI  = flux;


%% Prepare flux for Inhibitor

flux = zeros(20);

flux(2,1)  = x(1);   % rapid to HE1
flux(1,1)  =-x(37);  % rapid to urine

flux(17,1) = x(38);  % rapid to distribution
flux(1,17) = x(39);  % distribution to rapid

flux(12,13) = x(33); % Intestinal transit to intestine
flux(2,12)  = x(32); % intestine to Liver1

fBCLintKp  = x(34)/x(35) * 0.2;
fluxHEHC = [-fBCLintKp, 0; 0, 0];

flux(2:3,  2 :3 ) = fluxHEHC; 
flux(4:5,  4 :5 ) = fluxHEHC; 
flux(6:7,  6 :7 ) = fluxHEHC; 
flux(8:9,  8 :9 ) = fluxHEHC; 
flux(10:11,10:11) = fluxHEHC; 
flux(4 ,2 ) = x(1)/x(35);
flux(6 ,4 ) = x(1)/x(35);
flux(8 ,6 ) = x(1)/x(35);
flux(10,8 ) = x(1)/x(35);
flux(1,10 ) = x(1)/x(35); % Liver to rapid

fluxInhi = flux;


%% Combine flux
z = zeros(20,20);

flux = [fluxCont,z,z; z,fluxDDI,z; z,z,fluxInhi];


end


%% Prepare fluxmatrix with u
function flux = calcFluxMid(x,t,u,flux) %#ok<INUSL>

R_PSinf  = x(30)./(x(30) + u(42:2:50)/x(35));

flux(23, 22) = flux(23, 22) * R_PSinf(1);
flux(25, 24) = flux(25, 24) * R_PSinf(2);
flux(27, 26) = flux(27, 26) * R_PSinf(3);
flux(29, 28) = flux(29, 28) * R_PSinf(4);
flux(31, 30) = flux(31, 30) * R_PSinf(5);

% Put -(sum(outflow)) at flux(k,k) to keep mass balance
fluxDiag=sum(flux - diag(diag(flux)),1);
flux = flux - diag(fluxDiag);

end











%% %%%%%%%%%%%%% Do not modify below  %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [t,u] = calc_SepTP(fode, options, num_comp, timePoints,doseSin,doseInf)
global x doseInfusion_calcEach fluxPre odeStopped %kpInv

%% Prepare flux w/o cells containing u
fluxPre = calcFluxPre(x);

uinitial = zeros(1,num_comp);

% Separate time points
[tIni,tIniEraseIndex,timePointsSep,doseSingle_calc,doseInfusion_calc] = ...
    sepTimePoints(doseSin, doseInf, x, num_comp, timePoints);

% Initialize
tcombine = []; ucombine = [];

% Calculate ODE for each separated range
for k = 1: size(timePointsSep,2)

    % Initial setting for each cycle
    tp = timePointsSep{k};
    twoTimePoints = 0;
    
    if k>1, uinitial = uend; end
    
    uinitial = uinitial + doseSingle_calc(k,:);% Add infusion rate
    
    % If contain only two time points
    if size(timePointsSep{k},2) == 2
        twoTimePoints = 1;
        tp = [tp(1) (tp(1)+tp(2))/2 tp(2)];
    end
    
    % Decide calculate status
    calcStat = (sum(abs(uinitial))>0) + 2*(sum(abs(doseInfusion_calc(k,:)))>0);

    switch calcStat
        case 0 % No drug administered. Return zero
            t = tp';
            u = zeros(size(tp,2), num_comp);
        case 1 % W/O infusion
            [t,u]=fode(@RHS, tp, uinitial', options);
        case {2,3} % With infusion
            doseInfusion_calcEach = doseInfusion_calc(k,:);
            [t,u]=fode(@RHS_infusion, tp, uinitial', options);
    end
    if t(end) ~= tp(end), odeStopped = 1; break; end
    
    if twoTimePoints, t(2) = []; u(2,:) = []; end
    
    % Erase end point except for final loop
    if k <size(timePointsSep,2)
        uend =  u(end,:);
        t(end) = []; u(end,:) = [];
    end
    t = t + tIni(k);
    
    
    if tIniEraseIndex(k)
        t(1) = []; u(1,:) = [];
    end
    
    
    tcombine = [tcombine;t]; %#ok<AGROW> 
    ucombine = [ucombine;u]; %#ok<AGROW>
end

if odeStopped
    t = [-8,t(end)];
    u = ones(1, num_comp) * Inf;
else
    t = tcombine;
    u = ucombine;
end



end

%% Interface of ODE function: Normal administration
function [hReturn]=RHS(t,u)
global fluxPre volumeInv x startTime odeStopped odeStopTime

%['RHS0: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u; % Calculate differentials
hReturn = hReturn .* volumeInv; % Divide with volumes
%save('hReturn')

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end



%% Interface of ODE function: Complicated administration
function [hReturn]=RHS_infusion(t,u)
global fluxPre volumeInv x doseInfusion_calcEach startTime odeStopped odeStopTime

%['RHS1: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u;% Calculate differentials
hReturn = hReturn .* volumeInv;% Divide with volumes 
hReturn = hReturn + doseInfusion_calcEach';% Add infusion rate
%hReturn'

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end




%%
function [value,isterminal,direction] = events(t,u) %#ok<INUSD>
global odeStopped

value = odeStopped - 1;
isterminal = 1;
direction = 0;

end
