function [tIni,tIniEraseIndex,timePointsSep,doseSingle_calc,doseInfusion_calc] = ...
    sepTimePoints(doseSingle, doseInfusion, x, num_comp, timePoints) %#ok<INUSL>

%% Input
if size(doseSingle,2) == 1
    num_single = 0;
else
    num_single = size(doseSingle,1);
end
if size(doseInfusion,2) == 1
    num_infu = 0;
else
    num_infu   = size(doseInfusion,1);
end

singleComp = zeros(1,num_single);
singleTime = zeros(1,num_single);
singleDose = zeros(1,num_single);
infuComp   = zeros(1,num_infu);
infuTimeS  = zeros(1,num_infu);
infuDur    = zeros(1,num_infu);
infuDose   = zeros(1,num_infu);


for k = 1:num_single
    singleComp(k) = eval(doseSingle{k,1});
    singleTime(k) = eval(doseSingle{k,2});
    singleDose(k) = eval(doseSingle{k,3});
end
for k = 1:num_infu
    infuComp( k)   = eval(doseInfusion{k,1});
    infuTimeS(k)   = eval(doseInfusion{k,2});
    infuDur(  k)   = eval(doseInfusion{k,3});
    infuDose( k)   = eval(doseInfusion{k,4});
end
if find(infuDur == 0)
    error('infusion duration reached 0')
else
    infuTimeE = infuTimeS + infuDur;
    infuRate  = infuDose ./ infuDur;
end

tIni = unique(sort([0 singleTime infuTimeE infuTimeS]));


%% Evaluate values

% Erase tlag if tIni >= t(end)
tIni(tIni >= max(timePoints)) = [];


% Prepare time points and erase index for calculation
timePointsSep = cell(1,size(tIni,2));
tIniEraseIndex    = zeros(1,size(tIni,2));


% Separate time points with tIni
for k1 = 2:size(tIni,2)
    timePointsSep{k1-1} = [timePoints(timePoints < tIni(k1)) tIni(k1)];
    timePoints          =  timePoints(timePoints >=tIni(k1));
end
timePointsSep{end} = timePoints;


% Subtract tIni values
for k1=2:size(tIni,2)
    timePointsSep{k1} = timePointsSep{k1} - tIni(k1);
    
    % Modify if time points do not contain tlag value at head
    if timePointsSep{k1}(1)~=0
        timePointsSep{k1}     = [0 timePointsSep{k1}];
        tIniEraseIndex(k1) = 1;
    end
end




%% Calculate single dose
doseSingle_calc = zeros(size(tIni,2),num_comp);

for k1=1:size(tIni,2)
    indComp = find(tIni(k1) == singleTime);
    doseSingle_calc(k1,singleComp(indComp)) = singleDose(indComp);
end



%% Calculate infusion rates
doseInfusion_calc = zeros(size(tIni,2),num_comp);

for k1=1:size(tIni,2)
    indComp = find((tIni(k1)>=infuTimeS) .* (tIni(k1)<infuTimeE));
    doseInfusion_calc(k1,infuComp(indComp)) = infuRate(indComp);    
end



end





%% Test code
function testSeparateTimePoints %#ok<DEFNU>
%
%

x = [0.5,1,1];

timePoints = [0 1 2 3 4 5 6 7 8];
num_comp = 6;

%% Single administration
% {#compartment[int], dose time[str], dose amount[str]}

doseSingle = {...
    1,'x(1)'   ,'x(2)';...
    1,'x(1)+3' ,'x(2)';...
    2,'0'      ,'x(2)';...
    3,'x(1)'   ,'x(3)';...
    };


%% Infusion
% {#compartment[int], start time[str], duration[str], dose amount[str]}

doseInfusion = {...
    4,'0' ,'1',  '100';...
    5,'1' ,'1.5','150';...
    };

[tIni,tIniEraseIndex,timePointsSep,doseSingle_calc,doseInfusion_calc] = ...
    sepTimePoints(doseSingle, doseInfusion, x, num_comp,timePoints) %#ok<NASGU,ASGLU,NOPRT>

end