function [t,u] = ...
    sODE_didb_TC_clint(fode,x_in,timePoints,jacobian_index,num_comp,doseSin,doseInf)
%SOLVEODE Solve ode
% Add explanation here

% x_in
% jacobian_index

%% Compartments
%% 1-10: Parent 
%  1 Rapid
%  2 Intestine
%  3 Liver
%  4 Peripheral
%  5 Urine
%  6 Intestinel Transit
%
%% 11-40: Metabolites 1-3 
%  1 Rapid
%  2 Intestine
%  3 Liver
%  4 Peripheral
%  5 Urine
%
%% 41-46: Inhibitor
%  1 Rapid
%  2 Intestine
%  3 Liver
%  4 Peripheral
%  5 Urine
%  6 Intestinel Transit

%% Parameters definitions
global x volumeInv startTime odeStopped odeStopTime

warning('off','MATLAB:ode15s:IntegrationTolNotMet')

startTime = tic; 
odeStopped = 0;
odeStopTime= 15; % sec

x=(x_in);

x = tfCLint(x);

%CALCULATE inverse of compartment volumes
volumeInv = ones(50,1);
volumeInv( 1:10:41) = 1./x([4,41,51,61,91]); % Rapid
volumeInv( 3:10:43) = 1./x(2);            % Liver

volumeInv = [volumeInv; volumeInv];       % Control & DDI groups

%% Avoid division with zero
denomiParaInd = [2,4,7,41,51,61,43,53,63,91,94];
if sum(x(denomiParaInd)==0)
    error(['Divide with zero: #parameter(s) = ',...
        num2str(denomiParaInd(x(denomiParaInd)==0))])
end


%% Main calculation
options=odeset('Events',@events,'JPattern', jacobian_index);

[t,u] = calc_SepTP(fode, options, num_comp, timePoints,doseSin,doseInf);

%% Correct MW

u(:,[11:20,61:70]) = u(:,[11:20,61:70]) * x(81);
u(:,[21:30,71:80]) = u(:,[21:30,71:80]) * x(82);
u(:,[31:40,81:90]) = u(:,[31:40,81:90]) * x(83);

end



function x = tfCLint(x)

%Change CLint,total and ratios into each CLint

clCYP = x(31) * [x(28:30),1] / sum([x(28:30),1]);

x(16:4:28) = clCYP(1) * [x(16:4:24),1] / sum([x(16:4:24),1]);
x(17:4:29) = clCYP(2) * [x(17:4:25),1] / sum([x(17:4:25),1]);
x(18:4:30) = clCYP(3) * [x(18:4:26),1] / sum([x(18:4:26),1]);
x(19:4:31) = clCYP(4) * [x(19:4:27),1] / sum([x(19:4:27),1]);

end




%% Prepare fluxmatrix w/o u
function flux = calcFluxPre(x,t) %#ok<INUSD>
global kIntesElim

%% Parent
flux = zeros(10);

flux(3,1) = x(1);      % Rapid to Liver
flux(1,3) = x(1)/x(8); % Liver to Rapid
flux(4,1) = x(9);      % Rapid to Peripheral
flux(1,4) = x(10);     % Peripheral to Rapid
flux(5,1) = x(11);     % Rapid to Urine

kIntesElim = x(5) * (1-x(7))/ x(7);

flux(2,6) = x(6);      % Intes Transit to Intes
flux(3,2) = x(5);      % Intes to Liver
flux(2,2) =-kIntesElim / (1+sum(x(12:14))); % Intes to Feces

fluxPare = flux;


%% Metabolites

fluxMet1 = metaboliteFlux(x(41:50),x(1));
fluxMet2 = metaboliteFlux(x(51:60),x(1));
fluxMet3 = metaboliteFlux(x(61:70),x(1));



%% Inhibitor

flux = zeros(10);

flux(3,1) = x(1);       % Rapid to Liver
flux(1,3) = x(1)/x(95); % Liver to Rapid
flux(3,3) =-x(99)/x(95);% Liver to elimination
flux(4,1) = x(96);      % Rapid to Peripheral
flux(1,4) = x(97);      % Peripheral to Rapid
flux(5,1) = x(98);      % Rapid to Urine

flux(2,6) = x(93);      % Intes Transit to Intes
flux(3,2) = x(92);      % Intes to Liver
flux(2,2) =-x(92) * (1-x(94))/ x(94) ;% Intes to Feces

fluxInhi = flux;


%% Combine flux

z = zeros(10);

fluxCon = [...
    fluxPare, z,z,z,z;...
    z,fluxMet1, z,z,z;...
    z,z,fluxMet2, z,z;...
    z,z,z,fluxMet3, z;...
    z,z,z,z,fluxInhi ;...
    ];
fluxDDI = [...
    fluxPare, z,z,z,z;...
    z,fluxMet1, z,z,z;...
    z,z,fluxMet2, z,z;...
    z,z,z,fluxMet3, z;...
    z,z,z,z,z ;...
    ];




%% Combine and Put -(sum(outflow)) at flux(k,k) to keep mass balance

z = zeros(50);
flux = [fluxCon,z; z,fluxDDI];
    
fluxDiag=sum(flux - diag(diag(flux)),1);
flux = flux - diag(fluxDiag);

end




function flux = metaboliteFlux(x,qh)

flux = zeros(10);


flux(3,2) = x(2);     % Intes to Liver

flux(3,1) = qh;       % Rapid to Liver
flux(4,1) = x(4);     % Rapid to Peripheral
flux(5,1) = x(6);     % Rapid to Urine

flux(1,3) = x(1)/x(3);% Liver to Rapid
flux(1,4) = x(5);     % Peripheral to Rapid

end




%% Prepare fluxmatrix with u
function flux = calcFluxMid(x,t,u,fluxPre) %#ok<INUSL>
global kIntesElim

fluxCon = zeros(50);
fluxDDI = zeros(50);


%% Add metabolism for control group

kIntesArr = x(12:15) * kIntesElim / (1+sum(x(12:15)));
kIntesArr(4) = -kIntesArr(4);


fluxCon = addMetabolism(fluxCon,x,kIntesArr);


%% Add metabolism for DDI group
% Parent
kIntesArr     = kIntesArr  /(1+x(75)); % Intestine

if x(71) > 0
    x(16:4:31)    = x(16:4:31) /(1+u(43)/x(95)/x(71)); % Enzyme 1
    x(47:10:70)   = x(47:10:70)/(1+u(43)/x(95)/x(71)); % Enzyme 1
end
if x(72) > 0
    x(17:4:31)    = x(17:4:31) /(1+u(43)/x(95)/x(72)); % Enzyme 2
    x(48:10:70)   = x(48:10:70)/(1+u(43)/x(95)/x(72)); % Enzyme 2
end
if x(73) > 0
    x(18:4:31)    = x(18:4:31) /(1+u(43)/x(95)/x(73)); % Enzyme 3
    x(49:10:70)   = x(49:10:70)/(1+u(43)/x(95)/x(73)); % Enzyme 3
end
if x(74) > 0
    x(     31)    = x(     31) /(1+u(43)/x(95)/x(74)); % Enzyme Other
end


fluxDDI = addMetabolism(fluxDDI,x,kIntesArr);



%% Combine and Put -(sum(outflow)) at flux(k,k) to keep mass balance

z = zeros(50);
flux = [fluxCon,z; z,fluxDDI];

fluxDiag=sum(flux - diag(diag(flux)),1);
flux = flux - diag(fluxDiag);


flux = flux + fluxPre;

end





function flux = addMetabolism(flux,x,kIntesArr)

flux([12,22,32,2],2) = kIntesArr;% Intes to Met 1-3 Intes
flux(13,3) = sum(x(16:19))/x(8); % Liver to Met 1 Liver
flux(23,3) = sum(x(20:23))/x(8); % Liver to Met 2 Liver
flux(33,3) = sum(x(24:27))/x(8); % Liver to Met 3 Liver
flux( 3,3) =-sum(x(28:31))/x(8); % Liver to Met other

flux(13,13)=-sum(x(47:50))/x(43);% Met 1 Hepatic elimination
flux(23,23)=-sum(x(57:60))/x(53);% Met 2 Hepatic elimination
flux(33,33)=-sum(x(67:70))/x(63);% Met 3 Hepatic elimination

end





%% %%%%%%%%%%%%% Do not modify below  %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [t,u] = calc_SepTP(fode, options, num_comp, timePoints,doseSin,doseInf)
global x doseInfusion_calcEach fluxPre odeStopped %kpInv

%% Prepare flux w/o cells containing u
fluxPre = calcFluxPre(x);

uinitial = zeros(1,num_comp);

% Separate time points
[tIni,tIniEraseIndex,timePointsSep,doseSingle_calc,doseInfusion_calc] = ...
    sepTimePoints(doseSin, doseInf, x, num_comp, timePoints);

% Initialize
tcombine = []; ucombine = [];

% Calculate ODE for each separated range
for k = 1: size(timePointsSep,2)

    % Initial setting for each cycle
    tp = timePointsSep{k};
    twoTimePoints = 0;
    
    if k>1, uinitial = uend; end
    
    uinitial = uinitial + doseSingle_calc(k,:);% Add infusion rate
    
    % If contain only two time points
    if size(timePointsSep{k},2) == 2
        twoTimePoints = 1;
        tp = [tp(1) (tp(1)+tp(2))/2 tp(2)];
    end
    
    % Decide calculate status
    calcStat = (sum(abs(uinitial))>0) + 2*(sum(abs(doseInfusion_calc(k,:)))>0);

    switch calcStat
        case 0 % No drug administered. Return zero
            t = tp';
            u = zeros(size(tp,2), num_comp);
        case 1 % W/O infusion
            [t,u]=fode(@RHS, tp, uinitial', options);
        case {2,3} % With infusion
            doseInfusion_calcEach = doseInfusion_calc(k,:);
            [t,u]=fode(@RHS_infusion, tp, uinitial', options);
    end
    if t(end) ~= tp(end), odeStopped = 1; break; end
    
    if twoTimePoints, t(2) = []; u(2,:) = []; end
    
    % Erase end point except for final loop
    if k <size(timePointsSep,2)
        uend =  u(end,:);
        t(end) = []; u(end,:) = [];
    end
    t = t + tIni(k);
    
    if tIniEraseIndex(k)
        t(1) = []; u(1,:) = [];
    end
    
    
    tcombine = [tcombine;t]; %#ok<AGROW> 
    ucombine = [ucombine;u]; %#ok<AGROW>
end

if odeStopped
    t = [-8,t(end)];
    u = ones(1, num_comp) * Inf;
else
    t = tcombine;
    u = ucombine;
end



end

%% Interface of ODE function: Normal administration
function [hReturn]=RHS(t,u)
global fluxPre volumeInv x startTime odeStopped odeStopTime

%['RHS0: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u; % Calculate differentials
hReturn = hReturn .* volumeInv; % Divide with volumes
%save('hReturn')

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end



%% Interface of ODE function: Complicated administration
function [hReturn]=RHS_infusion(t,u)
global fluxPre volumeInv x doseInfusion_calcEach startTime odeStopped odeStopTime

%['RHS1: t = ' num2str(floor(t))]

flux = calcFluxMid(x,t,u,fluxPre); % Calculate flux with u

hReturn = flux * u;% Calculate differentials
hReturn = hReturn + doseInfusion_calcEach';% Add infusion rate
hReturn = hReturn .* volumeInv;% Divide with volumes 
%hReturn'

if toc(startTime) > odeStopTime
    odeStopped = odeStopped+1; 
end

end




%%
function [value,isterminal,direction] = events(t,u) %#ok<INUSD>
global odeStopped

value = odeStopped - 1;
isterminal = 1;
direction = 0;

end