function callmainLM1to5

%{
model='PRVRIF_130606/1_fixB_fixR';
mainLM1to5(model)
model='PRVRIF_130606/2_freeB_fixR';
mainLM1to5(model)
model='PRVRIF_130606/3_fixB_freeR';
mainLM1to5(model)
model='PRVRIF_130606/4_freeB_freeR';
mainLM1to5(model)
%}

model='PRVRIF_noEHC_130613/1_fixB_fixR';
mainLM1to5(model)
model='PRVRIF_noEHC_130613/2_freeB_fixR';
mainLM1to5(model)
model='PRVRIF_noEHC_130613/3_fixB_freeR';
mainLM1to5(model)
model='PRVRIF_noEHC_130613/4_freeB_freeR';
mainLM1to5(model)

end

function mainLM1to5(model)

%% Settings

cd('setting');
Setting = struct;
Setting.setFile = 'setting_LM1to5.m';

Setting = setting_LM1to5(Setting);
Setting = setComputerType(Setting);

Setting.calcType = 'mainLMauto';

Setting.Model = model;

%% Parameter and folder preparation

[Setting,calcFolder,resultFolderParent,calcID] = ...
    prepareFolder(Setting); %#ok<ASGLU,NASGU>
Setting.resultFolderParent = resultFolderParent;
cd('../');


%% 1to5: Substrate
[x_out(:,1),AIC(1)] = callMainLMeach(Setting,1.5,'/1','1-1com/');
[x_out(:,2),AIC(2)] = callMainLMeach(Setting,1,'/1','1-2com/');

% Use Vb, ka, and Lag from the result with lower AIC
select = find(AIC == min(AIC));
if numel(select)>1, select = select(1); end
Setting.kalag = x_out(2:3, select)';


[x_out2,~,SS] = callMainLMeach(Setting,2,'/2','2/');
Setting.parSubs = selectSample(x_out2,SS)';


%% 6to10: Inhibitor
[x_out(:,1),AIC(1)] = callMainLMeach(Setting,3.5,'/3','3-1com/');
[x_out(:,2),AIC(2)] = callMainLMeach(Setting,3,'/3','3-2com/');

% Use Vb, ka, and Lag from the result with lower AIC
select = find(AIC == min(AIC));
if numel(select)>1, select = select(1); end
Setting.kalag = x_out(2:3, select)';


[x_out4,~,SS] = callMainLMeach(Setting,4,'/4','4/');
Setting.parInhi = selectSample(x_out4,SS)';


%% 11 - PBPK - DDI

[x_out5,~,SS]= callMainLMeach(Setting,5,'/5','5/'); %#ok<ASGLU,NASGU>


%% CreateFig

cd('analyze')
createFigLM1to5(resultFolderParent,2)
cd('../')

%% Close

matlabpool size, if ans ~= 0, matlabpool close; end %#ok<NOANS>
if strcmp(computer('arch'),'glnxa64')
    %quit
end


end




function [x_out,AIC,exportSS] = callMainLMeach(Setting,condNum,modelAdd,foldAdd)


if nargin == 1
    Setting.Model  = [Setting.Model];
    resultFolder   = [SettingParent.resultFolderParent];
elseif nargin == 4
    Setting.Model  = [Setting.Model modelAdd];
    resultFolder   = [Setting.resultFolderParent, foldAdd];
    mkdir(resultFolder)
end


cd('import');
switch condNum
    case {1,1.5, 2, 3,3.5, 4}
        parameters=...
            str2double(importStr(['../../Model/', Setting.Model,'/',...
            'Parameters_LM.csv'],','))';
        
        switch condNum
            case {1.5,3.5}, 
                parameters(:,5:6) = 0;
            case {2,4}
                parameters(1:end-3,2:3) = ...
                    repmat(Setting.kalag,size(parameters(1:end-3,:),1),1);
        end
        
    case 5
        parSubs=...
            str2double(importStr(['../../Model/', Setting.Model,'/',...
            'Parameters_LM_subs.csv'],','))';
        parInhi=...
            str2double(importStr(['../../Model/', Setting.Model,'/',...
            'Parameters_LM_inhi.csv'],','))';
        
        parSet = [parSubs(end-2:end,:),parInhi(end-2:end,:)];
        parSubs = Setting.parSubs;
        parInhi = Setting.parInhi;
        
        sizeS = size(parSubs);
        sizeI = size(parInhi);
        
        parameters = zeros(sizeS(1)*sizeI(1), sizeS(2)+sizeI(2));
        parameters = [parameters; parSet];
        
        % Substrate
        subsInd = 1:sizeI(1):sizeS(1)*sizeI(1);
        for k = 0:sizeI(1)-1
            parameters(subsInd+k, 1:sizeS(2)) = parSubs;
        end
        
        % Inhibitor
        for k = 0:sizeS(1)-1
            parameters(1+k*sizeI(1):(k+1)*sizeI(1),sizeS(2)+1:end) = parInhi;
        end
        
end

cd('../');


csvwrite(['../Model/', Setting.Model,'/','Parameters_LM.tmp.csv'], parameters')


cd('calc');
[x_out,AIC,saveSS] = mainLMeach(resultFolder,Setting,parameters);
cd('../');


if nargout > 2
    exportSS = saveSS;
end

end




function nextPar = selectSample(x_out,SS)

x_out = x_out(:, SS<(min(SS)*10));
SS = SS(SS<min(SS)*10);

num_x = size(x_out,1);
k = 1;
while k <= size(x_out,2)
    xoNorm = zeros(size(x_out));
    for k1 = 1:size(x_out,2)
        xoNorm(:,k1) = x_out(:,k1) ./ x_out(:,k);
    end
    xoNorm(isnan(xoNorm)) = 1;
    xoNorm(xoNorm == Inf) = 0;
    tenPerDiff = (sum((0.9<=xoNorm).*(xoNorm<=1.1),1) == num_x);
    notSSmin = (SS > min(SS(tenPerDiff)));
    
    eracceInd = tenPerDiff .* notSSmin;
    x_out(:, eracceInd==1) = [];
    SS(eracceInd==1) = [];
    k=k+1;
end

nextPar = x_out;


end
