function createFig(dispLiver)

if nargin==0
    dispLiver = 0; 
    calcType = 'createFig';
elseif dispLiver == 1
    calcType = 'createFigLiver';
else
    error('Wrong input value')
end

tic;

%% Settings 
Setting = struct;

Setting = importSetting(Setting);
Model = [Setting.Model '/' Setting.ModelNum];

Numbers = ...
  struct('extrapolateAUCthres', 1/1000,'extrapolateAUCinf', 3,'sepAUCIndex',4); 

%prepare imported parameter and Jacobian
cd('import');
[ForODE,Numbers,InitialXY,~] = ...
  importParameters(Model, Numbers, calcType);
ForODE.odeSolvers = Setting.odeSolvers;
cd('../');

%% Calculation
uCell = cell(1,Numbers.samp);

cd('calc');

for k = 1:Numbers.samp
    k
    InitialXY.x = InitialXY.xArray(k,:);
    uCell{k} = PBPK_model_simulation(InitialXY, Numbers, ForODE);
end
cd('../');

uCell{1}

timePoints2 = Numbers.timePoints;
timePoints2(1) = [];

figure(1)

%% Combine u values for figure
for k = 1:Numbers.samp
    u = uCell{k};
    for k1 = 1:Numbers.timeFitComp
        uFig(:,k,k1) = u(:,Numbers.actTimeCourse(1,k1+1)); %#ok<AGROW>
    end
    
    if dispLiver
        for k1 = 1:Numbers.timeFitComp
            uFigLiver(:,k,k1) = u(:,Numbers.liverComp(1,k1)); %#ok<AGROW>
        end
    end
end

%% Plot
uFig

if dispLiver
    legendLabel = cell(Numbers.samp * 2 + 1,1);
    for k = 1:Numbers.samp
        legendLabel{k} = ['#',num2str(k)];
        legendLabel{k + Numbers.samp} = ['#',num2str(k)];
    end
    
    legendLabel{Numbers.samp * 2 + 1} = 'Observed';
    
else
    legendLabel = cell(Numbers.samp + 1,1);
    for k = 1:Numbers.samp
        legendLabel{k} = ['#',num2str(k)];
    end
    legendLabel{Numbers.samp+1} = 'Observed';
    
end
figure(1)

lineColorMap = winter(Numbers.samp+1);
%lineColorMap = lineColorMap(end:-1:1,:);

set(gcf, 'DefaultAxesColorOrder', lineColorMap)

for k1 = 1:Numbers.timeFitComp
  subplot(1,Numbers.timeFitComp,k1);
  semilogy(Numbers.timePoints, uFig(:,:,k1), ...
      'LineWidth', 1.5);%, 'LineColor', [0,0,0]);
  hold on
  
  if dispLiver
      plot(Numbers.timePoints, uFigLiver(:,:,k1), '--',...
          'LineWidth', 1.5);%, 'LineColor', [0,0,0]);
  end
  
  actTimeCourse = Numbers.actTimeCourse(:,k1+1);
  actTimeCourse(1) = [];
  plot(timePoints2, actTimeCourse, 'mo', ...
      'MarkerEdgeColor', [1,0,0], 'MarkerFaceColor', [1,0.5,0],...
      'LineWidth', 1.2, 'MarkerSize', 10);
  
  axis tight
  hleg1 = legend(legendLabel);
  
  xlim(Numbers.xlim)
  ylim(Numbers.ylim)
  set(gca,'FontSize',14);
  
  hold off
end

set(figure(1),'position', [150 50 600 600]);
saveas(figure(1),'fig/result.png')
saveas(figure(1),'fig/result.fig')


%{
%% Plot - old algorithm

% Simulated line
for k1 = 1:Numbers.timeFitComp
  u = uCell{1};
  subplot(1,Numbers.timeFitComp,k1);
  semilogy(Numbers.timePoints, u(:,Numbers.actTimeCourse(1,k1+1)));
  hold on
end

for k = 2:Numbers.samp
    % Simulated line
    for k1 = 1:Numbers.timeFitComp
        u = uCell{k};
        subplot(1,Numbers.timeFitComp,k1);
        plot(Numbers.timePoints, u(:,Numbers.actTimeCourse(1,k1+1)));
    end
end

% Actual line
for k1 = 1:Numbers.timeFitComp
  subplot(1,Numbers.timeFitComp,k1);
  actTimeCourse = Numbers.actTimeCourse(:,k1+1);
  actTimeCourse(1) = [];
  plot(timePoints2, actTimeCourse, 'mo');       
  xlim(Numbers.xlim)
  ylim(Numbers.ylim)
  
  hleg1 = legend('1','1','1','1');
end
%}

%% Close
matlabpool size, if ans ~= 0, matlabpool close; end %#ok<NOANS>

tocf(6)
end


function Setting = importSetting(Setting)

cd('import');
fid=fopen('../settingFig.txt');
settingFile = textscan(fid,'%s', 'delimiter', '', 'MultipleDelimsAsOne',0);
fclose(fid);
cd('../');

settingFile = settingFile{1};

for k = 1:3
    eval(settingFile{k})
end

end