function h = createFigLM(FigSet,uFig,isSortSS,num_disp,ssfold)
% h = createFigLM(FigSet,uFig,1)
% isSortSS == 1 - Select num_disp samples with low SS
% isSortSS == 2 - Select samples with SS < min(SS)*ssfold

if nargin == 2,num_disp=FigSet.parDiv; isSortSS=0; end
if nargin == 3,num_disp=FigSet.parDiv; ssfold = 10; end

parDiv = FigSet.parDiv;
saveSS = FigSet.saveSS;
x_out  = FigSet.x_out;

Numbers.actTimeCourse = FigSet.actTimeCourse;
Numbers.timeFitComp   = FigSet.timeFitComp;
Numbers.timePoints    = FigSet.timePoints;
Numbers.xlim          = FigSet.xlim;
Numbers.ylim          = FigSet.ylim;
num_tfc               = FigSet.timeFitCompNum;


timePoints2 = Numbers.timePoints;


%% Sort with SS
if isSortSS == 1
    [~, ssIndex] = sort(saveSS);
    ssIndex = ssIndex(1:num_disp);
    parDivDisp = num_disp;
    uFig = uFig(:,ssIndex,:);
    
elseif isSortSS == 2
    [uFig,ssIndex] = selectSample(x_out,saveSS,uFig,ssfold);
    parDivDisp = numel(ssIndex);
    
else
    ssIndex = 1:parDiv;
    parDivDisp = parDiv;
end


%% Figure preparation
legendLabel = cell(parDivDisp + 1,1);
for k = 1:parDivDisp
    legendLabel{k} = ['#',num2str(ssIndex(k))];
end
legendLabel{parDivDisp+1} = 'Obs.';

%% Reorder samples
% Ref: Reorder legend entires
% http://www.mathworks.se/matlabcentral/newsreader/view_thread/159423

lsGroupNum = 4; lsSampNum = ceil((parDivDisp)/lsGroupNum); forFigIndInv = [];
for k = 1:lsSampNum
    forFigIndInv = [forFigIndInv, k:lsSampNum:parDivDisp]; %#ok<AGROW>
end

forFigIndInv = forFigIndInv(1:parDivDisp);
[temp,forFigInd] = sort(forFigIndInv); %#ok<ASGLU>

for k1 = 1:num_tfc
    uFig(:,:,k1) = uFig(:,forFigInd,k1);
end


%% Create figure

h = figure(1);
clf(h,'reset');

S = get(0, 'ScreenSize');
lineColorMap = winter( max( ceil(parDivDisp/4), 1) );
set(h,'position', [S(3)-950 100 800 600]);
set(0, 'DefaultAxesColorOrder', lineColorMap,...
    'DefaultAxesLineStyleOrder', '-|--|-.|:')


for k1 = 1:num_tfc  
    subplot(1,num_tfc,k1);
    
    uFigEach = uFig(:,:,k1);
    
    if size(uFigEach,2) > 0
        semilogy(Numbers.timePoints', uFigEach,'LineWidth', 1.5);
    end
    hold on
    
    actTimeCourse = Numbers.actTimeCourse(:,k1);
    plot(timePoints2, actTimeCourse, 'mo', ...
        'MarkerEdgeColor', [1,0,0], 'MarkerFaceColor', [1,0.5,0],...
        'LineWidth', 1.2, 'MarkerSize', 10);
    
    % [left bottom width height]
    set(gca,'Position',[0.9/num_tfc*(k1-1)+0.07,0.05,0.9/num_tfc-0.07,0.9]);
    set(gca,'FontSize',20);
 
    if k1 ~= 1
       set(gca,'yticklabel',[]);
    end
    if k1 == num_tfc
        c = findobj(get(gca, 'Children'),'type','line');
        c = c(end:-1:1);
        hleg = legend(c([forFigIndInv,end]),legendLabel);
        set(hleg,'FontSize',10);
        pos = get(hleg,'position');
        set(hleg,'Position',[(1+pos(1))/2,pos(2:4)]);
    end
    
    xlim(Numbers.xlim)
    ylim(Numbers.ylim)
    
    hold off
end


end




function [uFig,ssInd] = selectSample(x_out,SS,uFig,ssfold)

ssInd = 1:numel(SS);

x_out = x_out(:,SS<(min(SS)*ssfold));
uFig  = uFig(:, SS<(min(SS)*ssfold),:);
ssInd = ssInd(SS<min(SS)*ssfold);
SS    = SS(SS<min(SS)*ssfold);

num_x = size(x_out,1);
k = 1;
while k <= size(x_out,2)
    xoNorm = zeros(size(x_out));
    for k1 = 1:size(x_out,2)
        xoNorm(:,k1) = x_out(:,k1) ./ x_out(:,k);
    end
    xoNorm(isnan(xoNorm)) = 1;
    xoNorm(xoNorm == Inf) = 0;
    tenPerDiff = (sum((0.9<=xoNorm).*(xoNorm<=1.1),1) == num_x);
    notSSmin = (SS > min(SS(tenPerDiff)));
    
    eracceInd = tenPerDiff .* notSSmin;
    x_out(:, eracceInd==1)  = [];
    uFig(:,  eracceInd==1,:)= [];
    ssInd(:, eracceInd==1)  = [];
    SS(eracceInd==1)        = [];
    k=k+1;
end


end