function [ForODE,Numbers,IniXY,ParaInd] = importParameters(Model,Numbers,IniXY,ForODE,ParaInd,calcType)

% Algorithm
ForODE.calcAlgorithm = importStr(['../../Model/',Model, '/','algorithm.txt'], ',')';

% Numbers
[Numbers,isAutoCalcGoal]=setNumbers(Numbers,ForODE,Model,ForODE.calcAlgorithm(1),calcType);

% Calculation type specific values
switch calcType
    case 'main'
        [Numbers,ParaInd,IniXY] = setParaMain(Numbers,ParaInd,IniXY,Model);
    case 'simulation'
        [Numbers,ParaInd,IniXY] = setParaSimu(Numbers,ParaInd,IniXY,Model);
    case {'mainLM','mainLMauto'}
        [Numbers,ParaInd,IniXY] = setParaLM(Numbers,ParaInd,IniXY,Model);
    otherwise, error('Wrong calcType name')
end


%% Prepare ODE or Laplace equations
switch char(ForODE.calcAlgorithm(1))
    %case {'ode','odegpu'}, [Numbers,ForODE] = prepODE(Numbers,ForODE,Model);
    case 'ode_mfile',      [Numbers,ForODE] = prepODEm(Numbers,ForODE,Model);   
    case 'FILT',           [Numbers,ForODE] = prepFILT(Numbers,ForODE,Model);  
    case 'OE',                      ForODE  = prepOE(ForODE);
    otherwise, error('Wrong algorithm name')
end


%% Prepare y_out equations & values
switch calcType
    case {'mainLM','mainLMauto'}
        Numbers.goal = numel(Numbers.timeFitComp);
        
        Numbers.u_goal = 0;
        for k = 1:Numbers.goal
            u1 = Numbers.actTimeCourse(:,k);
            u1((u1==0),:) = [];
            Numbers.u_goal = Numbers.u_goal + numel(u1);
        end
        
    otherwise
        switch char(ForODE.calcAlgorithm(1))
            case {'ode','odegpu','ode_mfile','FILT'}
                [IniXY,Numbers,ForODE] = ...
                    prepYstarEq(IniXY,Numbers,ForODE,Model,isAutoCalcGoal);
            case 'OE'
                IniXY.y_star = importYstar(Model);
                Numbers.goal = numel(IniXY.y_star);
        end
        y_out = IniXY.y_star; %#ok<NASGU>
end

end




function [Numbers,isAutoCalcGoal]=setNumbers(Numbers,ForODE,Model,calcAlgorithm,calcType)


NumbersRead  = csvread(['../../Model/',Model, '/','Numbers.txt']);
Numbers.sepCond= NumbersRead(1);  % #conditions
isAutoCalcGoal = NumbersRead(2);
Numbers.xlim   = NumbersRead(3:4);
Numbers.ylim   = NumbersRead(5:6);

Numbers.acc_y_folddiff = 0.1;   % acceptable y difference

switch char(calcAlgorithm)
    case {'ode','odegpu','ode_mfile','FILT'}
        switch calcType
            case {'main','simulation'}
                
                if  ForODE.useTemp
                    tmpSuff = '.tmp';
                else
                    tmpSuff = '';
                end
                
                actTC_import = ...
                    csvread(['../../Model/',Model, '/','ActualTimeCourse',...
                    tmpSuff,'.csv']);
            case {'mainLM','mainLMauto'}
                actTC_import = ...
                    csvread(['../../Model/',Model, '/','ActualTimeCourse_LM.csv']);
        end
        
        Numbers = checkActTC(actTC_import,Numbers);
        
        if isequal(Numbers.xlim,[0,0])
            Numbers.xlim = [0,Numbers.timePoints(end)];
        end
end

end




function Numbers = checkActTC(actTC_import,Numbers)

Numbers.timePoints    = actTC_import(2:end,1)';
Numbers.actTimeCourse = actTC_import(2:end,2:end); 
Numbers.timeFitComp   = actTC_import(1,2:end);        
Numbers.timeFitCompNum= size(actTC_import,2) - 1;

if isequal(  unique(Numbers.timePoints),Numbers.timePoints) == false
    error('ActualTimeCourse should not have duplicate data on any time points')
elseif isequal(sort(Numbers.timePoints),Numbers.timePoints) == false
    error('ActualTimeCourse must have time points in ascending order')
elseif Numbers.timePoints(1) < 0
    error('ActualTimeCourse should not have negative time points')
end


if Numbers.timePoints(1)
    Numbers.timePoints = [0,Numbers.timePoints];
    Numbers.actTimeCourse = ...
        [zeros(1,size(Numbers.actTimeCourse,2));Numbers.actTimeCourse];
    
    Numbers.actTC_addZero = 1;
else
    Numbers.actTC_addZero = 0;
end

end



function [Numbers,ParaInd,IniXY] = ...
    setParaMain(Numbers,ParaInd,IniXY,Model)

%% Import and modify parameters
parameters = str2double(importStr(['../../Model/',Model, '/','Parameters.csv'], ','))';
parameters = modifyParaMain(parameters,ParaInd);


%% Get parameter settings

ParaInd.free= find(parameters(3,:)~=0);
ParaInd.fix = find(parameters(3,:)==0);
ParaInd.min = find(parameters(3,ParaInd.free)==2|parameters(3,ParaInd.free)==4);
ParaInd.max = find(parameters(3,ParaInd.free)==3|parameters(3,ParaInd.free)==4);

Numbers.xfree= size(ParaInd.free,2);
Numbers.xfix = size(ParaInd.fix,2);
Numbers.xmin = size(ParaInd.min,2);
Numbers.xmax = size(ParaInd.max,2);
Numbers.input= Numbers.xfree+Numbers.xfix;

if Numbers.xmax > 0
    error(['Maximum limitation is '...
        'not supported yet due to diffuculty in parameter transformation'])
end


ParaInd.normalDist= find(parameters(6,ParaInd.free)~=1);
ParaInd.expDist   = find(parameters(6,ParaInd.free)==1);
ParaInd.divPar    = find(parameters(7,ParaInd.free)>1);
IniXY.xDivNum     = parameters(7,ParaInd.free);
ParaInd.divParExp = find(parameters(6,ParaInd.free(ParaInd.divPar))==1);

IniXY.xfix    = parameters(1,ParaInd.fix);
IniXY.xfree   = parameters(1:2,ParaInd.free);
IniXY.freelim = parameters(4:5,ParaInd.free);

if find((IniXY.xfree(1,ParaInd.min)-IniXY.freelim(1,ParaInd.min)) <= 0)
        error('Lower limit(s) must be smaller than xmin of initial distribution')
end
if find((IniXY.xfree(2,ParaInd.max)-IniXY.freelim(1,ParaInd.max)) >= 0)
        error('Higher limit(s) must be larger than xmax of initial distribution')
end

if IniXY.strictLowLim
    IniXY.xfree(1:2,ParaInd.min) = ...
        IniXY.xfree(1:2,ParaInd.min)-repmat(IniXY.freelim(1,ParaInd.min),2,1);
end

IniXY.xfree(1:2,ParaInd.expDist) = log(IniXY.xfree(1:2,ParaInd.expDist));

end



%% Modify parameters for DIDB analyses
function parameters = modifyParaMain(parameters,ParaInd)

if ParaInd.noMetabolite == 1
    parameters(2:end,[41:70,91:95]) = 0; % Fix parameters for metabolites
    parameters(:,16:27)     = 0; % Set CLs  to form metabolites = 0
    parameters(:,12:14)     = 0; % Set kmet to form metabolites = 0
elseif ParaInd.noMetabolite == 2
    parameters(2:end,[41:70,91:95]) = 0; % Fix parameters for metabolites
    parameters(:,[16:18,20:22,24:26,28:30])     = 0; % Set CLs  to form metabolites = 0
    parameters(:,12:14)     = 0; % Set kmet to form metabolites = 0
end

end




function [Numbers,ParaInd,IniXY] = ...
    setParaSimu(Numbers,ParaInd,IniXY,Model)

% Import parameters
IniXY.xArray = ...
    str2double(importStr(['../../Model/',Model, '/','Parameters_simulation.csv'], ','))';
Numbers.samp = size(IniXY.xArray,1);
Numbers.input = size(IniXY.xArray,2);

end




function [Numbers,ParaInd,IniXY] = ...
    setParaLM(Numbers,ParaInd,IniXY,Model)

% Import parameters
parameters = csvread(['../../Model/',Model, '/','Parameters_LM.tmp.csv'])';
parameters = parameters([Numbers.kParDiv,(end-2):end],:);

% Get free and fix parameters
ParaInd.free  = find(parameters(2,:)~=0);
ParaInd.fix   = find(parameters(2,:)==0);

xfree      = parameters(1,ParaInd.free); %initial parameters
IniXY.xfix = parameters(1,ParaInd.fix);  %parameters for fixed

Numbers.xfree = size(ParaInd.free,2);
Numbers.xfix  = size(ParaInd.fix,2);
Numbers.input = Numbers.xfree+Numbers.xfix;


% Get parameter limitations

ParaInd.min = find(parameters(2,ParaInd.free)==2|parameters(2,ParaInd.free)==4);
ParaInd.max = find(parameters(2,ParaInd.free)==3|parameters(2,ParaInd.free)==4);

freelimits = zeros(2,Numbers.xfree);
freelimits(1,:) =-Inf;
freelimits(2,:) = Inf;

freelimits(1,ParaInd.min) = parameters(3,ParaInd.free(ParaInd.min));
freelimits(2,ParaInd.max) = parameters(4,ParaInd.free(ParaInd.max));

% Parameter transformation
pindExp = find(parameters(2,ParaInd.free)==2);
pindSin = find(parameters(2,ParaInd.free)==4);

if numel(find(parameters(2,ParaInd.free)==3))
    error(['Maximum limitation only is '...
        'not supported yet due to diffuculty in parameter transformation'])
end

xfree(1,pindExp)=log(xfree(1,pindExp) - freelimits(1,pindExp));
%xfree(1,pindSin)=asin(2*(xfree(1,pindSin)-freelimits(1,pindSin))...
%    ./(freelimits(2,pindSin)-freelimits(1,pindSin))-1);
limPlus = freelimits(2,pindSin) + freelimits(1,pindSin);
limMinus= freelimits(2,pindSin) - freelimits(1,pindSin);
xfree(1,pindSin) = asin((2*xfree(1,pindSin) - limPlus) ./ limMinus);

% Export
IniXY.xfree      = xfree;
IniXY.freelimits = freelimits;
ParaInd.exp   = pindExp;
ParaInd.sin   = pindSin;

end





function [Numbers,ForODE] = prepODEm(Numbers,ForODE,Model)

% Import compartment #
num_comp=importStr(['../../Model/',Model, '/','num_comp.txt'], ',')';
Numbers.compAll = zeros(1,Numbers.sepCond);

for k = 1:Numbers.sepCond
    Numbers.compAll(k) = str2double(num_comp{k});
end

% Import dose
Numbers.doseSin = importStr(['../../Model/',Model, '/','dose_single.csv'], ',');
Numbers.doseInf = importStr(['../../Model/',Model, '/','dose_infusion.csv'], ',');

% Import m-file for ODE
ForODE.mfuncAll = cell(1,Numbers.sepCond);
for k = 1:Numbers.sepCond
    mfunc = importStr(['../../Model/',Model, '/','mfunc' num2str(k) '.txt'], ',');
    ForODE.mfuncAll{k} = eval(mfunc{1});
end

cd(['../../Model/',Model])
ForODE.jacobian_indexAll = makeJacobianIndex;
%!pwd
numSlash = numel([strfind(Model,'/'),strfind(Model,'\')]);
cdDest = '../../src/import';
for k = 1:numSlash, cdDest = ['../',cdDest];end %#ok<AGROW>

cd(cdDest)

end




function ForODE = prepOE(ForODE)

ForODE.OE = cell(1,Numbers.sepCond);
for k = 1:Numbers.sepCond
    eq = importStr(['../../Model/',Model, '/','OE' num2str(k) '.txt'], '@');
    ForODE.OE{k }= eq;
end
end




function [Numbers,ForODE] = prepFILT(Numbers,ForODE,Model)

for k = 1:Numbers.sepCond
    Numbers.compAll(k) = 1;
end
Numbers.timePoints2 = Numbers.timePoints;
Numbers.timePoints2(Numbers.timePoints2 == 0) = [];
Numbers.timeNum = size(Numbers.timePoints2,2);

Numbers.approxErr= 6; %approximation error, A
Numbers.truncErr = 6; %truncation errorm, K
Numbers.EulerSum = 6; %P
Numbers.expApproxErr=exp(Numbers.approxErr); %EX

ForODE.FILTAll = cell(1,Numbers.sepCond);
for k = 1:Numbers.sepCond
    ForODE.FILTAll{k}= ...
        importStr(['../../Model/',Model, '/','FILT' num2str(k) '.txt'], ',');
end

FILT_ar = Numbers.approxErr./Numbers.timePoints2;
FILT_ii = pi./Numbers.timePoints2;
Numbers.FILT_ax = Numbers.expApproxErr./Numbers.timePoints2;

Numbers.FILT_Matrix = sparse(zeros(size(Numbers.timePoints2,2)*2));
Numbers.FILT_iiMatrix = sparse(zeros(size(Numbers.timePoints2,2)*2));
for k = 1:size(Numbers.timePoints2,2)
    Numbers.FILT_Matrix(k*2-1,k*2-1) = FILT_ar(k);
    Numbers.FILT_Matrix(k*2,k*2) = FILT_ar(k);
    Numbers.FILT_iiMatrix(k*2,k*2-1) = FILT_ii(k);
    Numbers.FILT_iiMatrix(k*2-1,k*2) = -FILT_ii(k);
end
Numbers.FILT_Matrix = Numbers.FILT_Matrix + Numbers.FILT_iiMatrix/2;


end




function [IniXY,Numbers,ForODE] = ...
    prepYstarEq(IniXY,Numbers,ForODE,Model,isAutoCalcGoal)

%% y_out equation
% Will cause error if timepoints(2) == 0
% Need to correct later

ForODE.y_outCompNumEachCond = cell(1,Numbers.sepCond);
ForODE.y_outEqAll =  cell(1,Numbers.sepCond);
y_outEqForAutoCalc = [];
Numbers.goal = 0;
NumCompInPrevCond = 0;


if  ForODE.useTemp
    tmpSuff = '.tmp';
else
    tmpSuff = '';
end

for k = 1:Numbers.sepCond
    youtImport=importStr(['../../Model/',Model,'/','y_outEq',num2str(k),...
        tmpSuff,'.txt'], ')')';
    
    youtImport(strcmp(youtImport(:),'0'))=[];
    youtImport{1} = [youtImport{1} ')'];
    for k1 = 2:numel(youtImport)
        youtImport{k1} = [youtImport{k1}(2:end) ')'];
    end
    
    ForODE.y_outEqAll{k} = youtImport;
    y_outEqForAutoCalc = [y_outEqForAutoCalc; youtImport]; %#ok<AGROW>
    Numbers.goalEach(k) = numel(ForODE.y_outEqAll{k});
    
    %Prepare for y_out calculation
    for k1 = 1:Numbers.goalEach(k)
        Numbers.goal = Numbers.goal + 1;
        parenthesis1 = strfind(youtImport(k1), '(');
        parenthesis2 = strfind(youtImport(k1), ')');
        parenthesis1 = parenthesis1{1};
        parenthesis2 = parenthesis2{1};
        
        % If Y_star contain u
        commaIndex = strfind(youtImport(k1), ',');
        if commaIndex{1}, parenthesis1 = commaIndex{1}; end
        
        compNum = str2double(youtImport{k1}((parenthesis1 + 1):(parenthesis2 - 1)));
        
        ForODE.y_outCompNumEachCond{k} = ...
            [ForODE.y_outCompNumEachCond{k} (compNum - NumCompInPrevCond)];
        ForODE.y_outCompNumAll(Numbers.goal) = compNum;
        
        %Separate equations with parenthesis
        ForODE.y_outEqAll{k}(k1) = cellstr(youtImport{k1}(1:parenthesis1));
        y_outEqForAutoCalc(Numbers.goal) = cellstr(youtImport{k1}(1:parenthesis1));
    end
    NumCompInPrevCond = NumCompInPrevCond + Numbers.compAll(k);
end


%% Calculate objective values

u = zeros(size(Numbers.timePoints,2),Numbers.goal);
Numbers.tCalcInd = cell(1,Numbers.goal);
for k = 1:Numbers.goal
    Numbers.tCalcInd{k} = 1:numel(Numbers.timePoints);
end

if isAutoCalcGoal == 1
    IniXY.y_star = [];
    %Calculate goals
    for k = 1:Numbers.goal
        u(:,k) = Numbers.actTimeCourse(:, ...
            Numbers.timeFitComp == ForODE.y_outCompNumAll(k));
        Numbers.tCalcInd{k} = unique([1;find(u(:,k)~=0)]);
    end
    
    cd('../calc');
    y_out = calcY_stars(Numbers.timePoints',u,y_outEqForAutoCalc,Numbers);
    cd('../import');
    
    IniXY.y_star = y_out;
else
    IniXY.y_star = csvread(['../../Model/',Model, '/','y_star',tmpSuff,'.csv']);
    
    if numel(IniXY.y_star) ~= Numbers.goal
        IniXY.y_star
        Numbers.goal
        error('Size differs between equation and target value');
    end
end

end
