function mainCal(divID,Setting)

calcType = 'main';

%% Matlabpool
if (strcmp(computer('arch'),'glnxa64'))
    'maxNumCompThreads in mainCal' %#ok<NOPRT>
    maxNumCompThreads
    %if ans~=1, maxNumCompThreads(int8(getenv('NCPUS'))-48),end %#ok<NOANS>
end

%% Setting
resultFolder = Setting.resultFolder;
Model        = Setting.Model;
if exist(['../Model/',Model],'dir') ~= 7
    error(['No designated model files: ',Model])
end

tic;


%% Prepare folders to export data
mkdir([resultFolder 'fig/']);



%% Prepare parameters
Numbers= struct('extrapolateAUCthres', 1/1000,...
    'extrapolateAUCinf', 3,...
    'sepAUCIndex',4);
IniXY  = struct('strictLowLim', Setting.strictLowLim);
ForODE = struct('useTemp',  Setting.useTemp);
ParaInd= struct('noInhOtherPath',Setting.noInhOtherPath,...
    'noMetabolite',  Setting.noMetabolite);

cd('import');
[ForODE,Numbers,IniXY,ParaInd] = importParameters(Model,Numbers,IniXY,ForODE,ParaInd,calcType);
ForODE.odeSolvers    = Setting.odeSolvers;
cd('../');


Numbers.iter     = Setting.iter;
Numbers.samp     = Setting.samp;
Numbers.deltaForS= Setting.deltaForS;
Numbers.samp     = Numbers.samp*(Setting.isDivSampRand+1);

OtherSet = struct;
OtherSet.isSaveYdist    = Setting.isSaveYdist;
OtherSet.isDivSampRand  = Setting.isDivSampRand;
OtherSet.isDivSampDeltaS= Setting.isDivSampDeltaS;
OtherSet.deltaForS      = Setting.deltaForS;

save([resultFolder,'Structs'],'IniXY','ForODE','Numbers','ParaInd','OtherSet')


if nargin
    IniXY.xfree(:,ParaInd.divPar) = Setting.xfreeDivArray;
end

y_star = IniXY.y_star; save([resultFolder 'y_star'], 'y_star');
csvwrite([resultFolder '/y_star.csv'], y_star);

if strcmp(computer('arch'),'maci64')==0
    switch char(ForODE.calcAlgorithm(1))
        case {'ode','odegpu','ode_mfile','FILT'}
            matlabpool size, if ans == 0, matlabpool open; end %#ok<NOANS>
    end
end

%% Calculation

cd('parameterPrep');

if Setting.calcSet == 6
    IniXY = initialDistribution_beta(IniXY, Numbers,Setting);
else
    IniXY = initialDistribution(IniXY, Numbers);
end

cd('../calc');
resultFolder2 = resultFolder;

[FinalXforCalculation,l_a,stopCalc,errState,tcell1,ucell1,X,Y_k_half,sortedY,sortWithY]=...
    twiceLSmethod(Model, resultFolder2, IniXY, ...
    Numbers, ForODE, ParaInd, OtherSet, divID); %#ok<ASGLU>

if stopCalc ~=1
    
    mkdir([resultFolder,'finalXY/'])
    
    FinalXsortY = FinalXforCalculation(:,sortWithY)';
    FinalYsortY = Y_k_half(:,sortWithY)';
    csvwrite([resultFolder 'finalXY/FinalXsortY.csv'], FinalXsortY')
    csvwrite([resultFolder 'finalXY/FinalYsortY.csv'], FinalYsortY')
    save([resultFolder 'finalXY/FinalXYsortY'],'FinalXsortY','FinalYsortY')
    
    
    if Setting.onlyAUC == 0
        [sumlogdiffAve,FinalXsorted] = ...
            compareTimecourse(@PBPK_model, @PBPK_model2, Model, resultFolder2,...
            FinalXforCalculation, Numbers, ForODE, tcell1, ucell1, Y_k_half, divID); %#ok<ASGLU>
        
        finalX = FinalXsorted(:,ParaInd.free)';
        yellowNumFin = [];
    else
        finalX = FinalXforCalculation(ParaInd.free,:);
        yellowNumFin = 0;
    end
    
    
    size(finalX)
    initX  = X(:,:,1)';
    initX(:,ParaInd.expDist) =exp(initX(:,ParaInd.expDist));
    
    if IniXY.strictLowLim
        initX(:,ParaInd.min) = ...
            initX(:,ParaInd.min) + ...
            repmat(IniXY.freelim(1,ParaInd.min),Numbers.samp,1);
    end
    
    cd('../analyze');
    if Setting.saveFm == 1
        if Setting.onlyAUC
            saveFm(FinalXsortY,resultFolder,Setting.calcTypeList,Setting.saveFm);
        else
            saveFm(FinalXsorted,resultFolder,Setting.calcTypeList,Setting.saveFm);
        end
    end
    
    parDistCmp(initX',finalX,[],1,0,0,[-5,5,1],yellowNumFin)
    saveas(figure(6),[resultFolder 'fig/distX.png'])
    parDistCmp(initX',finalX,[],1,1,0,[-5,5,1],yellowNumFin)
    saveas(figure(6),[resultFolder 'fig/distXnorm.png'])
    cd('../');
    
else
    cd('../');
    errmsg = errmsglist(errState);
    
    fid = fopen([resultFolder 'errState_' num2str(errState) '.txt'], 'wt');
    fprintf(fid,'%s', errmsg); fclose(fid); %Save error state
end

%% Close
csvwrite([resultFolder 'tocf.csv'], tocf(6))
tocf(6)
end




function errmsg = errmsglist(errState)


switch errState
    case 1, errmsg = 'y_out of all samples reached Inf';
    case 2, errmsg = 'RCOND too low';
    case 3, errmsg = 'RCOND equals NaN';
    case 4, errmsg = 'Parameter reached Inf';
    case 5, errmsg = 'No samples in parameter ranges';
    case 6, errmsg = 'Errors in FILT calculation';
    case 7, errmsg = 'Parameter values too large (>10^10)';
    case 8, errmsg = 'ODE solver stopped';
end

end