function [t_out,uCell_out] = simulation(isDefaultLineColor)
%% prepare matlab

tic;
calcType = 'simulation';

if nargin == 0
   isDefaultLineColor = 0; 
end

%% Settings

cd('setting');
Setting = struct;
Setting = setting_simulation(Setting);
cd('../');

Model=Setting.Model;

if exist(['../Model/',Model],'dir') ~= 7
   error(['No designated model files: ',Model]) 
end

%% prepare parameters
resultFolder = ['../results_simulation/' Model '/' datestr(now, 'yyyymmdd-HHMM') '/']; %#ok<NASGU>
%mkdir(resultFolder);

Numbers = ...
    struct('extrapolateAUCthres', 1/1000,'extrapolateAUCinf', 3,'sepAUCIndex',4);
IniXY  = struct;
ForODE = struct('useTemp',  Setting.useTemp);
ParaInd= struct;

%prepare imported parameter and Jacobian
cd('import');
[ForODE,Numbers,IniXY,~] = importParameters(Model,Numbers,IniXY,ForODE,ParaInd,calcType);
cd('../');

%% Apply settings

ForODE.odeSolvers    = Setting.odeSolvers;
%ForODE.odeSolversGPU = Setting.odeSolversGPU;
%ForODE.dt = Setting.dt;

%% Calculation
Numbers, ForODE %#ok<NOPRT>

IniXY %#ok<NOPRT>
IniXY.xArray' %#ok<NOPRT>

uCell = cell(1,Numbers.samp);
Y_k_half = zeros(Numbers.samp,Numbers.goal);

xArray = IniXY.xArray;

if Numbers.samp > 1
    % matlabpool size, if ans==0, matlabpool open, end %#ok<NOANS>
end
%Numbers.sepCond

switch char(ForODE.calcAlgorithm(1))
    case {'ode','FILT','ode_mfile'}
        
        disp('Numbers.actTimeCourse = ')
        disp(Numbers.actTimeCourse)
        
        parfor ksamp= 1:Numbers.samp
            cd('calc');
            
            XforCalculation = xArray(ksamp,:)';
            
            uEach = [];
            Y_k_halfEach = [];
            for k2 = 1:Numbers.sepCond %#ok<PFBNS>
                
                [youtCalc,t,uCalc]=PBPK_model(XforCalculation, Model, Numbers, ForODE, 1, k2); %#ok<ASGLU>
                
                sizeuCalc = size(uCalc,1);
                if sizeuCalc<size(Numbers.timePoints,2)
                    uCalc = [uCalc; zeros(size(Numbers.timePoints,2) - sizeuCalc,Numbers.compAll(k2))];
                end
                
                uEach = [uEach uCalc]; 
                Y_k_halfEach = [Y_k_halfEach youtCalc];
                
            end
            
            uCell{ksamp} = uEach;
            Y_k_half(ksamp,:) = Y_k_halfEach;
            
            disp(num2str(ksamp))
            cd('../');
        end
        
        tocf(6)
       
        
        
        
        %% Combine u values for figure
        for ksamp = 1:Numbers.samp
            u = uCell{ksamp};
            for k1 = 1:Numbers.timeFitCompNum
                uFig(:,ksamp,k1) = u(:,Numbers.timeFitComp(1,k1)); %#ok<AGROW>
            end
        end
        
        
        %% output and close
        figure(1)
        
        legendLabel = cell(Numbers.samp+1,1);
        for ksamp = 1:Numbers.samp
            legendLabel{ksamp} = ['#',num2str(ksamp)];
        end
        legendLabel{Numbers.samp+1} = 'Observed'; %#ok<NASGU>
        
        figure(1)
        
        lineColorMap = winter(Numbers.samp+1);
        %lineColorMap = lineColorMap(end:-1:1,:);
        
        set(gcf, 'DefaultAxesColorOrder', lineColorMap);
        
        
        for k1 = 1:Numbers.timeFitCompNum
            subplot(1,Numbers.timeFitCompNum,k1);
            if isDefaultLineColor
                semilogy(Numbers.timePoints, uFig(:,:,k1), 'b-');
            else
                semilogy(Numbers.timePoints, uFig(:,:,k1), ...
                    'LineWidth', 1.5);%, 'LineColor', [0,0,0]);
            end
            
            hold on
            actTimeCourse = Numbers.actTimeCourse(:,k1);
            
            plot(Numbers.timePoints, actTimeCourse, 'mo', ...
                'MarkerEdgeColor', [1,0,0], 'MarkerFaceColor', [1,0.5,0],...
                'LineWidth', 1.2, 'MarkerSize', 10);
            
            axis tight
            %hleg1 = legend(legendLabel);
            
            xlim(Numbers.xlim)
            ylim(Numbers.ylim)
            set(gca,'FontSize',18);
            
            hold off
        end
        
        %uCell{1} %#ok<NOPRT>
        
        %{
for k1 = 1:Numbers.timeFitComp
  subplot(2,floor((Numbers.timeFitComp/2)+0.5),k1);
  semilogy(tcell{1}, ucell{1}(:,compIndex(k1)));
  hold on
  plot(Numbers.actTimeCourse(:,1), Numbers.actTimeCourse(:,k1+1), 'mo');
  ylim(Numbers.ylim)
  hold off
end
        %}
        %csvwrite([resultFolder 'tocf.csv'], tocf(6))
        
        
    case {'OE'}
        
        for ksamp= 1:Numbers.samp
            XforCalculation = xArray(ksamp,:);
            
            parfor k1=1:Numbers.samp
                Y_k_halfEach = [];
                for k2 = 1:Numbers.sepCond %#ok<PFBNS>
                    youtCalc = solveOE(XforCalculation(k1,:), ForODE, k2);
                    Y_k_halfEach = [Y_k_halfEach youtCalc];
                end
                Y_k_half(k1,:) = Y_k_halfEach';
            end
            
            tocf(6)
        end
end

exportY = [IniXY.y_star' Y_k_half'] %#ok<NASGU,NOPRT>
exportYdiv = diag(1./IniXY.y_star) * Y_k_half' %#ok<NASGU,NOPRT>
%csvwrite('exportY.csv', exportY)
%Ysum = sum(exportY,1);
%size(exportY)
%Ysum1 = sum(exportY(1:14,:),1)
%Ysum2 = sum(exportY(15:23,:),1)

if isDefaultLineColor
    set(gcf,'Position',[800,600,1000,300])
    hgexport(figure(1),'temp/result.png',hgexport('factorystyle'),'Format','png')
end

tocf(6)

if nargout == 2
    t_out     = Numbers.timePoints;
    uCell_out = uCell;
end

end
