function calcLogSS(FinalXsorted)

if strcmp(computer('arch'),'glnxa64')
  maxNumCompThreads(int8(getenv('NCPUS'))-48)
end

if strcmp(computer('arch'),'maci64')==0
    matlabpool size, if ans == 0, matlabpool open; end %#ok<NOANS>
end
  
%% Settings and Import

Model='PitCsA_120730'; 
calcType='main';
Setting.odeSolvers=  {@ode45 @ode15s @ode23s};

Numbers = ...
  struct('extrapolateAUCthres', 1/1000,'extrapolateAUCinf', 3,'sepAUCIndex',4); 

cd('import');
[ForODE Numbers InitialXY ParameterIndex] = importParameters(Model, Numbers, calcType);
ForODE.odeSolvers = Setting.odeSolvers;
cd('../');

Numbers.samp = size(FinalXsorted,1);

%% Calc

parfor k1=1:Numbers.samp
  k1
  cd('calc');
  Y_k_halfCalc = []; ufit = [];
  for k2 = 1:Numbers.sepCond %#ok<PFBNS>
    
    [youtCalc t uCalc]=...
      PBPK_model(FinalXsorted(k1,:),Model,Numbers,ForODE,1,k2); %#ok<ASGLU>
    
    sizeuCalc = size(uCalc,1);
    if sizeuCalc<size(Numbers.timePoints,2)
      uCalc = [uCalc; ...
        zeros(size(Numbers.timePoints,2)-sizeuCalc,Numbers.compAll(k2))];
    end
    
    ufit = [ufit uCalc];
    Y_k_halfCalc = [Y_k_halfCalc youtCalc];
  end
  
  Y_k_half(:,k1) = Y_k_halfCalc';
  tcell1{k1}=Numbers.timePoints';
  ucell1{k1}=ufit;
  cd('../');
end

%% Transform obtained u cells into an array
ufit2=zeros(size(Numbers.timePoints,2),sum(Numbers.compAll),Numbers.samp);
for k2 = 1:Numbers.samp
  calcTimes = size(ucell1{k2},1);
  ufit2(1:calcTimes,:,k2)=ucell1{k2};
end

ufit2(1,:,:)=[];

not_inf_index=find(sum(Y_k_half,1)~=Inf);
calc_index2 = not_inf_index;%calc_index(not_inf_index);

l_posi2=size(calc_index2,2) %#ok<NOPRT>
FinalX=FinalXsorted(not_inf_index,:);
FinalY=Y_k_half(:,not_inf_index);
ufit3=ufit2(:,:,not_inf_index);

tcell2=tcell1(not_inf_index);
ucell2=ucell1(not_inf_index);

%% calc differences
logSS = zeros(l_posi2,1);
compIndex =Numbers.actTimeCourse(1,:); compIndex(1)=[];
for k1=1:l_posi2
  for k2=1:Numbers.timeFitComp
    clear u1
    clear u2
    %combine actual and calculated value
    u1(:,1:2)=Numbers.actTimeCourse([2 end],[1 k2+1]); %observed
    u1(:,3)=ufit3(:,compIndex(k2),k1); %calculated
    
    %erase zero
    timeIndex=find(u1(:,2)~=0);
    sizetimeIndex=size(timeIndex,1);
    u2(1:sizetimeIndex,1:2)=u1(timeIndex,2:3);
    
    %calc difference
    logSS(k1) = logSS(k1) + sum((log(u2(:,1)./u2(:,2)).^2));
  end
end


%% sort data
[logSSsorted logSSIndex]=sort(logSS);
logSSBest = sum(logSSsorted(1));
FinalXsorted = FinalX(logSSIndex,:);

csvwrite('./Temp/logSSsorted.csv', logSSsorted)
save('./Temp/logSSsorted', 'logSSsorted')
csvwrite('./Temp/FinalXsorted.csv', FinalXsorted)
save('./Temp/FinalXsorted', 'FinalXsorted')

%{
FinalXsorted = FinalX(logSSIndex,:);
FinalYsorted = FinalY(:,logSSIndex)';
tcell = tcell2(logSSIndex,:);
ucell = ucell2(logSSIndex,:);
%}

end

function [resultFolder calcID] = prepareFolder(Model)

calcMethod = getCD;
cDate = datestr(now, 'yyyymmdd-HHMM');
calcID = cDate;

[commentSave commentHead] = saveComment;

isFoldExist = 1;

while isFoldExist
  resultFolder = ...
    ['../results/' cDir '/' Model '/' calcID '-' commentHead '/'];
  [~,~,messid] = mkdir(resultFolder);
  
  %Change folder name if the folder already exist
  if strcmp(messid,'MATLAB:MKDIR:DirectoryExists')==0
    isFoldExist = 0;
  else
    isFoldExist = isFoldExist + 1;
    calcID = [cDate num2str(isFoldExist)];
  end
end
%{
resultFolder = ...
  ['../results/' calcMethod '/' Model '/' ...
  calcID '-' commentHead '/'];
mkdir(resultFolder)
%}

fid = fopen([resultFolder commentHead '-comment.txt'], 'wt');
fprintf(fid,'%s\n', commentSave); fclose(fid);

transSet{1} = calcMethod;
transSet{2} = Model;%#ok<NASGU>
%transSet{3} = cDate;
%transSet{4} = commentHead;
%transSet{5} = commentSave; 

save([resultFolder 'transSet'], 'transSet')

end

function [commentSave commentHead] = saveComment

if strcmp(computer('arch'),'glnxa64')
  commentSave='t2'; commentHead='t2';
else
  commentSave = input('Comment?: ','s');
  if isempty(commentSave)
    commentHead = [];
  else
    blankIndex = strfind(commentSave, ' ');
    if isempty(blankIndex)
      commentHead = commentSave;
    else
      commentHead = commentSave(1:(blankIndex-1));
    end
  end
end

end

function cDir = getCD

cDir = cd;

switch computer('arch')
  case {'maci64','glnxa64'}
    cdSp = strfind(cDir,'/'); cDir = cDir(cdSp(end)+1:end) ;
  otherwise
    cdSp = strfind(cDir,'\'); cDir = cDir(cdSp(end)+1:end) ;
end

end
