function mainLM1to5

%% Settings

cd('setting');
Setting = struct;
Setting.setFile = 'setting_LM1to5.txt';

Setting = importSetting(Setting);
Setting = setComputerType(Setting);

Setting.calcType = 'mainLMauto';


%% Parameter and folder preparation

[Setting,calcFolder,resultFolderParent,calcID] = ...
    prepareFolder(Setting); %#ok<ASGLU,NASGU>
Setting.resultFolderParent = resultFolderParent;
cd('../');


%% 1to5: Substrate
[x_out(:,1),AIC(1)] = callMainLMeach(Setting,1,'/1','1-1com/');
[x_out(:,2),AIC(2)] = callMainLMeach(Setting,2,'/1','1-2com/');

% Use Vb, ka, and Lag from the result with lower AIC
select = find(AIC == min(AIC));
if numel(select)>1, select = select(1); end
Setting.vbkalag = x_out(1:3, select)';


[x_out4,~,exportSS] = callMainLMeach(Setting,4,'/4','4/');

%% Close

matlabpool size, if ans ~= 0, matlabpool close; end %#ok<NOANS>
if strcmp(computer('arch'),'glnxa64')
    %quit
end


end




function [x_out,AIC,exportSS] = callMainLMeach(Setting,condNum,modelAdd,foldAdd)


if nargin == 1
    Setting.Model  = [Setting.Model];
    resultFolder   = [SettingParent.resultFolderParent];
elseif nargin == 4
    Setting.Model  = [Setting.Model modelAdd];
    resultFolder   = [Setting.resultFolderParent, foldAdd];
    mkdir(resultFolder)
end


cd('import');
switch condNum
    case {1,2,4,5,6,7,9,10}
        parameters=...
            str2double(importStr(['../../Model/', Setting.Model,'/',...
            'Parameters_LM.csv'],','))';
        
        switch condNum
            case {1,6}, 
                parameters(:,5:6) = 0;
            case {4,9}
                parameters(1:end-3,1:3) = ...
                    repmat(Setting.vbkalag,size(parameters(1:end-3,:),1),1);
            case {5,10}
                parameters = [Setting.parNext;parameters(end-2:end,:)];
        end
        
    case 11
        parSubs=...
            str2double(importStr(['../../Model/', Setting.Model,'/',...
            'Parameters_LM_subs.csv'],','))';
        parInhi=...
            str2double(importStr(['../../Model/', Setting.Model,'/',...
            'Parameters_LM_inhi.csv'],','))';
        
        parSet = [parSubs(end-2:end,:),parInhi(end-2:end,:)];
        parSubs = Setting.parSubs;
        parInhi = Setting.parInhi;
        
        sizeS = size(parSubs);
        sizeI = size(parInhi);
        
        parameters = zeros(sizeS(1)*sizeI(1), sizeS(2)+sizeI(2));
        parameters = [parameters; parSet];
        
        % Substrate
        subsInd = 1:sizeI(1):sizeS(1)*sizeI(1);
        for k = 0:sizeI(1)-1
            parameters(subsInd+k, 1:sizeS(2)) = parSubs;
        end
        
        % Inhibitor
        for k = 0:sizeS(1)-1
            parameters(1+k*sizeI(1):(k+1)*sizeI(1),sizeS(2)+1:end) = parInhi;
        end
        
end

cd('../');


csvwrite(['../Model/', Setting.Model,'/','Parameters_LM.tmp.csv'], parameters')


cd('calc');
[x_out,AIC,saveSS] = mainLMeach(resultFolder,Setting,parameters);
cd('../');


if nargout > 2
    exportSS = saveSS;
end

end




function nextPar = selectSample(x_out,SS)

accSSfold = 100;

x_out = x_out(:, SS<(min(accSSfold)*10));
SS = SS(SS<min(accSSfold)*10);

num_x = size(x_out,1);
k = 1;
while k <= size(x_out,2)
    xoNorm = zeros(size(x_out));
    for k1 = 1:size(x_out,2)
        xoNorm(:,k1) = x_out(:,k1) ./ x_out(:,k);
    end
    xoNorm(isnan(xoNorm)) = 1;
    xoNorm(xoNorm == Inf) = 0;
    tenPerDiff = (sum((0.9<=xoNorm).*(xoNorm<=1.1),1) == num_x);
    notSSmin = (SS > min(SS(tenPerDiff)));
    
    eracceInd = tenPerDiff .* notSSmin;
    x_out(:, eracceInd==1) = [];
    SS(eracceInd==1) = [];
    k=k+1;
end

nextPar = x_out;


end
