function mainLMRfree

%% Settings

cd('setting');
Setting = struct;
Setting.setFile = 'setting_LMRfree.txt';

Setting = importSetting(Setting);
Setting = setComputerType(Setting);

Setting.calcType = 'mainLMauto';


%% Parameter and folder preparation

[Setting,calcFolder,resultFolderParent,calcID] = ...
    prepareFolder(Setting); %#ok<ASGLU,NASGU>
Setting.resultFolderParent = resultFolderParent;
cd('../');


%% 1 - Compartmental analysis -Inhibitor
[x_out(:,1),AIC(1)] = callMainLMeach(Setting,1,'/1','1-1com/');
[x_out(:,2),AIC(2)] = callMainLMeach(Setting,2,'/1','1-2com/');

% Use Vb, ka, and Lag from the result with lower AIC
select = find(AIC == min(AIC));
if numel(select)>1, select = select(1); end
Setting.vbkalag = x_out(1:3, select)';

%% 3 - PBPK - Inhibitor

[x_out3,~,SS] = callMainLMeach(Setting,3,'/3','3/');
Setting.parInhi = selectSample(x_out3,SS);
Setting.parInhi = Setting.parInhi';

%% 2 - Compartmental analysis -Substrate
[x_out(:,1),AIC(1)] = callMainLMeach(Setting,1,'/2','2-1com/');
[x_out(:,2),AIC(2)] = callMainLMeach(Setting,2,'/2','2-2com/');

% Use Vb, ka, and Lag from the result with lower AIC
Setting.vbkalag = x_out(1:3, AIC == min(AIC))';

%% 5 - PBPK - Substrate

[x_out4,~] = callMainLMeach(Setting,4,'/4','4/'); %#ok<ASGLU>


%% Close

matlabpool size, if ans ~= 0, matlabpool close; end %#ok<NOANS>
if strcmp(computer('arch'),'glnxa64')
    %quit
end


end




function [x_out,AIC,exportSS] = callMainLMeach(Setting,condNum,modelAdd,foldAdd)


if nargin == 1
    Setting.Model  = [Setting.Model];
    resultFolder   = [SettingParent.resultFolderParent];
elseif nargin == 4
    Setting.Model  = [Setting.Model modelAdd];
    resultFolder   = [Setting.resultFolderParent, foldAdd];
    mkdir(resultFolder)
end


cd('import');
switch condNum
    case {1,2,3}
        parameters=...
            str2double(importStr(['../../Model/', Setting.Model,'/',...
            'Parameters_LM.csv'],','))';
        
        if condNum==1, parameters(:,5:6) = 0;end
        if condNum==3
            parameters(1:end-3,1:3) = ...
                repmat(Setting.vbkalag,size(parameters(1:end-3,:),1),1);
        end
        
    case 4
        parSubs=...
            str2double(importStr(['../../Model/', Setting.Model,'/',...
            'Parameters_LM_subs.csv'],','))';
        parInhi=...
            str2double(importStr(['../../Model/', Setting.Model,'/',...
            'Parameters_LM_inhi.csv'],','))';
        
        parSet = [parSubs(end-2:end,:),parInhi(end-2:end,:)];
        parSubs = parSubs(1:end-3,:);
        parSubs(:,1:3) = repmat(Setting.vbkalag,size(parSubs,1),1);
        parInhi = Setting.parInhi; %parInhi(1:end-3,:);
        
        sizeS = size(parSubs);
        sizeI = size(parInhi);
        
        parameters = zeros(sizeS(1)*sizeI(1), sizeS(2)+sizeI(2));
        parameters = [parameters; parSet];
        
        % Substrate
        subsInd = 1:sizeI(1):sizeS(1)*sizeI(1);
        for k = 0:sizeI(1)-1
            parameters(subsInd+k, 1:sizeS(2)) = parSubs;
        end
        
        % Inhibitor
        for k = 0:sizeS(1)-1
            parameters(1+k*sizeI(1):(k+1)*sizeI(1),sizeS(2)+1:end) = parInhi;
        end
        
end

cd('../');


csvwrite(['../Model/', Setting.Model,'/','Parameters_LM.tmp.csv'], parameters')


[x_out,AIC,SS] = mainLMeach(resultFolder,Setting,parameters);

if nargout > 2
    exportSS = SS;
end

end




function nextPar = selectSample(x_out,SS)

x_out = x_out(:, SS<min(SS)*10);
SS = SS(SS<min(SS)*10);

num_x = size(x_out,1);
k = 1;
while k <= size(x_out,2)
    xoNorm = zeros(size(x_out));
    for k1 = 1:size(x_out,2)
        xoNorm(:,k1) = x_out(:,k1) ./ x_out(:,k);
    end
    xoNorm(isnan(xoNorm)) = 1;
    xoNorm(xoNorm == Inf) = 0;
    tenPerDiff = (sum((0.9<=xoNorm).*(xoNorm<=1.1),1) == num_x);
    notSSmin = (SS > min(SS(tenPerDiff)));
    
    eracceInd = tenPerDiff .* notSSmin;
    x_out(:, eracceInd==1) = [];
    SS(eracceInd==1) = [];
    k=k+1;
end

nextPar = x_out;


end
