function [t,u] = solveODE_GPU(fodeStr, Numbers, ForODE)
%SOLVEODE Solve ode with GPU
%

global Numbers2 ForODE2 x CLmatrixValues CompVolumes
global startTime isLongCalcTime longCalcSample odeStopped condNum2
global dt

x = Numbers.x_in;
startTime = tic;
isLongCalcTime = 0;
longCalcSample = Numbers.loopNumber;
odeStopped = 0;
condNum2 = Numbers.condNum;
[Numbers2,ForODE2] = setNumbersForODE(Numbers, ForODE, condNum2);
fode = str2func(fodeStr);
dt = ForODE.dt;

%% Prepare fluxmatrix w/o u()
CLmatrixValues = zeros(Numbers2.comp);

for k=1:Numbers2.FluxmatrixFirstCalcNum
    CLmatrixValues(ForODE2.FluxmatrixFirstCalcIndex(k)) = ...
        eval(char(ForODE2.Fluxmatrix(ForODE2.FluxmatrixFirstCalcIndex(k))));
end

%% Prepare compvolumes
CompVolumes = zeros(1,Numbers2.conc);
for k1=1:Numbers2.conc
    CompVolumes(1,k1)=...
        eval(char(ForODE2.volumes{condNum2}(ForODE2.isconc(k1))));
end

%% Prepare initial values w/o tlag
uinitial=zeros(1,Numbers2.comp);
for k1=1:Numbers2.nottlag
    uinitial(Numbers2.nottlagindex(k1)) = ...
        eval(char(ForODE2.initialamountEq{condNum2}(Numbers2.nottlagindex(k1))));
end

%% Select RHS
if Numbers2.complexDose==0
    RHS=@RHS0;
elseif numel(ForODE.infIndex{condNum2})==0
    RHS=@RHS0;
else
    RHS=@RHS1;
    for k=1:numel(ForODE2.infIndex{condNum2})
        ForODE2.infTotalDoseEval(k)=eval(ForODE2.infTotalDose{condNum2}{k});
        ForODE2.infDurEval(k)=eval(ForODE2.infDur{condNum2}{k});
    end
end

%% Main calculation
%options=odeset('Events',@events,'JPattern',ForODE2.jacobian_index);
if Numbers2.tlag==0
    [t,u]=fode(RHS, Numbers2.timePoints, uinitial');  %SOLVING ODE
else
    [tlag tlagforCalc tlagEraseIndex timePointsForCalc] = separateTimePoints(Numbers2, ForODE2, x, condNum2);
    
    tlagforCalc = [0 tlagforCalc];
    tcombine = []; ucombine = [];
    
    for k = 1: size(timePointsForCalc,2)
        %%% k
        
        tp = timePointsForCalc{k};
        twoTimePoints = 0;
        
        % Prepare initial values
        uinitialPlusIndex = Numbers2.tlagindex(tlag==tlagforCalc(k));
        if k>1
            uinitial = uend;
            for k1 = 1:size(uinitialPlusIndex,2)
                uinitial(uinitialPlusIndex(k1))= ...
                    uend(uinitialPlusIndex(k1)) + ...
                    eval(char(ForODE2.initialamountEq{condNum2}(uinitialPlusIndex(k1))));
            end
        end
        
        % If contain only two time points
        if size(timePointsForCalc{k},2) == 2
            twoTimePoints = 1;
            tp = [tp(1) (tp(1)+tp(2))/2 tp(2)];
        end
        
        %%% timePointsForCalc{k}
        
        % Calculation
        if sum(uinitial) ==0
            t = tp';
            u = zeros(size(tp,2),Numbers2.comp);
        else            
            [t,u]=fode(RHS, tp, uinitial');  %SOLVING ODE
        end
        if twoTimePoints, t(2) = []; u(2,:) = []; end
        
        % Erase end point except for final loop
        if k <size(timePointsForCalc,2)
            uend =  u(end,:);
            t(end) = []; u(end,:) = [];
        end
        t = t + tlagforCalc(k);
        
        if tlagEraseIndex(k)
            t(1) = []; u(1,:) = [];
        end
        
        tcombine = [tcombine;t']; %#ok<AGROW>
        ucombine = [ucombine;u]; %#ok<AGROW>
        
        %%% tcombine
    end
    
    t=tcombine;
    u=ucombine;
    
end



end


%%
function [hReturn]=RHS0(t,u)
%normal administration
global x ForODE2 CompVolumes

hReturn=h(x,u,t);

%devide with volumes
hReturn(ForODE2.isconc)=hReturn(ForODE2.isconc)./CompVolumes';

end

%%
function [hReturn]=RHS1(t,u)
%complicated administration
global x ForODE2 CompVolumes condNum2

hReturn=h(x,u,t);

for k=1:numel(ForODE2.infIndex{condNum2})
  if t <= ForODE2.infDurEval(k)
    hReturn(ForODE2.infIndex{condNum2}(k))=...
      hReturn(ForODE2.infIndex{condNum2}(k)) + ...
      ForODE2.infTotalDoseEval(k)./ForODE2.infDurEval(k);
  end
end

%devide with volumes
hReturn(ForODE2.isconc)=hReturn(ForODE2.isconc)./CompVolumes';

end

%%

function h=h(x,u,t) %#ok<INUSD,INUSL>

global Numbers2 ForODE2 CLmatrixValues 
global startTime isLongCalcTime longCalcSample odeStopped %#ok<NUSED>
%global Numbers2 CLmatrix2 Model2  ConcIndex2 CompImport2 

%calculate flux w/ u()
for k=1:Numbers2.FluxmatrixLaterCalcNum
  CLmatrixValues(ForODE2.FluxmatrixLaterCalcIndex(k)) = ...
    eval(char(ForODE2.Fluxmatrix(ForODE2.FluxmatrixLaterCalcIndex(k))));
end

%Calculate efflux to subtract
CLmatrixValues2 = CLmatrixValues;
CLmatrixdiag=sum(CLmatrixValues2 - diag(diag(CLmatrixValues2)));
CLmatrixValues2 = CLmatrixValues2 - diag(CLmatrixdiag);

%compute the right hand side
h=CLmatrixValues2 * u';

if toc(startTime) > 120
  if odeStopped == 0
    odeStopped=1; 
  end
end

end

%%
function [value,isterminal,direction] = events(t,u) %#ok<INUSD>
global odeStopped

value = 1-odeStopped;
isterminal = 1;
direction = 0;

end
function [Numbers2 ForODE2] = setNumbersForODE(Numbers, ForODE, condNum)

Numbers2 = Numbers;
ForODE2 = ForODE;

ForODE2.FluxmatrixFirstCalcIndex =  ForODE.FluxmatrixFirstCalcIndexAll{condNum};
ForODE2.FluxmatrixLaterCalcIndex =  ForODE.FluxmatrixLaterCalcIndexAll{condNum};
ForODE2.Fluxmatrix =  ForODE.FluxmatrixAll{condNum};
%ForODE2.y_outEq = ForODE.y_outEqAll{condNum};
ForODE2.jacobian_index = ForODE.jacobian_indexAll{condNum};
Numbers2.FluxmatrixFirstCalcNum = Numbers.FluxmatrixFirstCalcNumAll(condNum);
Numbers2.FluxmatrixLaterCalcNum = Numbers.FluxmatrixLaterCalcNumAll(condNum);
ForODE2.isconc = ForODE.isconcAll{condNum};
Numbers2.conc = Numbers.concAll(condNum);
Numbers2.comp = Numbers.compAll(condNum);
Numbers2.nottlagindex = Numbers2.nottlagindexAll{condNum};
Numbers2.tlagindex = Numbers2.tlagindexAll{condNum};
Numbers2.nottlag = Numbers2.nottlagAll(condNum);
Numbers2.tlag = Numbers2.tlagAll(condNum);

end


function [t u] = ode45_gpu(RHS, t, uinit) %#ok<DEFNU>
global dt

% Runge-Kutta-Fehlberg
epsilon = 0.0001;
tCalc = [0 dt:dt:t(end) t]; tCalc = unique(sort(tCalc));
%uCalc = zeros(numel(uinit), numel(tCalc)); 
uCalc = uinit;

butcher_t1 = [ ...
    1, 1/4,       0,         0, 0, 0; ...
    1, 3/32,      9/32,      0, 0, 0; ...
    1, 1932/2197,-7200/2197, 7296/2197,  0,         0; ...
    1, 439/216,  -8,         3680/513,  -845/4104,  0; ...
    1, -8/27,      2,        -3544/2565, 1859/4104,-11/40;...
    ]';
    
butcher_t2 = [ ...
    1, 25/216, 0, 1408/2565,  2197/4104,   -1/5,  0 ...
    1, 16/135, 0, 6656/12825, 28561/56430, -9/50, 2/55 ...
    ]';

tol_calVec = [1/360, 0, -128/4275, -2197/75240, 1/50, 2/55]';

kArray = zeros(size(uinit,2),6);

for k = 1:(numel(tCalc)-1)
    tolExceed = 1;
    
    while tolExceed == 1
        %tCalc(k)
        kArray(:,1)= RHS(tCalc(k), uCalc(k,:))' * dt;
        
        kArray(:,2)= RHS(tCalc(k)+dt/4,    ...
            [uCalc(:,end), kArray(:,1)]   * butcher_t1(1:2,1))' * dt;
        
        kArray(:,3)= RHS(tCalc(k)+dt*3/8,  ...
            [uCalc(:,end), kArray(:,1:2)] * butcher_t1(1:3,2))' * dt;
        
        kArray(:,4)= RHS(tCalc(k)+dt*12/13,...
            [uCalc(:,end), kArray(:,1:3)] * butcher_t1(1:4,3))' * dt;
        
        kArray(:,5)= RHS(tCalc(k)+dt,      ...
            [uCalc(:,end), kArray(:,1:4)] * butcher_t1(1:5,4))' * dt;
        
        kArray(:,6)= RHS(tCalc(k)+dt/2,    ...
            [uCalc(:,end), kArray(:,1:5)] * butcher_t1(1:6,5))' * dt;
        
        uCalc4 = [uCalc(:,end), kArray(:,1:5)] * butcher_t2(1:6,1);
        %uCalc5 = [uCalc(:,end), kArray(:,1:6)] * butcher_t2(1:7,1);
        
        diff45 = max(kArray * tol_calVec);
        s = (epsilon*dt/ (2*diff45))^0.25;
    end
    
    %% Add routine to decide continue or retry
    
    %%
    
    uCalc = [uCalc; uCalc4]; %#ok<AGROW>
end
uCalc = uCalc';

% Export
u = zeros(numel(t),numel(uinit));
for k = 1:numel(t)
    if find(tCalc == t(k))
        u(k,:) = uCalc(:,tCalc == t(k));
    else
        lowIndex = find(tCalc < t(k));
        u(k,:) = sum(uCalc(lowIndex(end):lowIndex(end)+1,:),1)/2;
    end
end

end

function [t u] = ode2_gpu(RHS, t, uinit) %#ok<DEFNU>
global dt

% 2th-order Runge-Kutta
tCalc = [0 dt:dt:t(end) t]; tCalc = unique(sort(tCalc));
uCalc = zeros(numel(tCalc),numel(uinit)); uCalc(1,:) = uinit';

for k = 1:(numel(tCalc)-1)
    %tCalc(k)
    k1 = RHS(tCalc(k),        uCalc(k,:))' * dt;
    k2 = RHS(tCalc(k) + dt/2, uCalc(k,:) + k1 /2)' * dt;

    uCalc(k+1,:) = uCalc(k,:) + k2;
end

% Export
u = zeros(numel(t),numel(uinit));
for k = 1:numel(t)
    u(k,:) = uCalc(tCalc == t(k),:);
end

end

function [t u] = ode4_gpu(RHS, t, uinit) %#ok<DEFNU>
global dt

% 4th-order Runge-Kutta
tCalc = [0 dt:dt:t(end) t]; tCalc = unique(sort(tCalc));
uCalc = zeros(numel(tCalc),numel(uinit)); uCalc(1,:) = uinit';

for k = 1:(numel(tCalc)-1)
    %tCalc(k)
    k1 = RHS(tCalc(k),        uCalc(k,:)   )' * dt;
    k2 = RHS(tCalc(k) + dt/2, uCalc(k,:) + k1 /2)' * dt;
    k3 = RHS(tCalc(k) + dt/2, uCalc(k,:) + k2 /2)' * dt;
    k4 = RHS(tCalc(k) + dt  , uCalc(k,:) + k3 )' * dt;

    uCalc(k+1,:) = uCalc(k,:) + (k1 + 2*k2 + 2*k3 + k4) / 6;
end

% Export
u = zeros(numel(t),numel(uinit));
for k = 1:numel(t)
    u(k,:) = uCalc(tCalc == t(k),:);
end

end