function submit
% Submitting job for CNM
%   Specify various parameters and separates parameter ranges

if strcmp(computer('arch'),'glnxa64')
  maxNumCompThreads(int8(getenv('NCPUS'))-48)
end
  
%% Settings 

Model='PitCsA_addAdipose_120906'; 

Setting = struct;

Setting.iter=   10; %overwrite numbers if needed
Setting.samp=   300; %overwrite numbers if needed

Setting.odeSolvers=  {@ode45 @ode15s @ode23s};

Setting.isUseInitialDistriDeltaS = 0;
Setting.deltaForS = 0.1;
Setting.isDivGroupsIter = 0;
Setting.isDivS = 1; % Use triangle division

Setting.isDivOverlap = 0; %0- no overlap and just separate, 1 - overlap
Setting.paramDivDirec = 0; % 0:ascending, 0:descending 

%% Prepare folders
[resultFolder calcID] = prepareFolder(Model);
copyfile(['../Model/',Model], [resultFolder 'input']);
copyfile(cd, [resultFolder 'scripts']);

%% Prepare parameters

cd('import');
parameters=str2double(importStr(['../../Model/',Model, '/','Parameters.csv'], ','))';
cd('../');

[InitialXY ParameterIndex] = impParForDiv(parameters);

cd('parameterPrep');
InitialXY.isDivOverlap  = Setting.isDivOverlap;
InitialXY.paramDivDirec = Setting.paramDivDirec;
[xfreeDivArray xfreeDivIndex xDivIsFit divideNumberProd]=...
  divParamRange(InitialXY, ParameterIndex, resultFolder); %#ok<ASGLU>
cd('../');

for k = 1:divideNumberProd
  parameters(1:2,ParameterIndex.free(ParameterIndex.divPar))=xfreeDivArray((k*2-1:k*2),:);
  mkdir([resultFolder num2str(k)])
  csvwrite([resultFolder num2str(k) '/Parameters.csv'], parameters)
end

if divideNumberProd>20
  'Parameter division# too large' %#ok<NOPRT>
  exit
end

if isT2
  [calcID divideNumberProd] %#ok<NOPRT>
  exit
else
  for k = 1:divideNumberProd
    cd([resultFolder 'scripts']);
    mainCal(calcID,k)
  end
end

end

function [resultFolder calcID] = prepareFolder(Model)

calcMethod = getCD;
cDate = datestr(now, 'yyyymmdd-HHMM');
calcID = cDate;

[commentSave commentHead] = saveComment;

isFoldExist = 1;

while isFoldExist
  resultFolder = ...
    ['../results/' cDir '/' Model '/' calcID '-' commentHead '/'];
  [~,~,messid] = mkdir(resultFolder);
  
  %Change folder name if the folder already exist
  if strcmp(messid,'MATLAB:MKDIR:DirectoryExists')==0
    isFoldExist = 0;
  else
    isFoldExist = isFoldExist + 1;
    calcID = [cDate num2str(isFoldExist)];
  end
end
%{
resultFolder = ...
  ['../results/' calcMethod '/' Model '/' ...
  calcID '-' commentHead '/'];
mkdir(resultFolder)
%}

fid = fopen([resultFolder commentHead '-comment.txt'], 'wt');
fprintf(fid,'%s\n', commentSave); fclose(fid);

transSet{1} = calcMethod;
transSet{2} = Model;%#ok<NASGU>
%transSet{3} = cDate;
%transSet{4} = commentHead;
%transSet{5} = commentSave; 

save([resultFolder 'transSet'], 'transSet')

end

function [commentSave commentHead] = saveComment

if strcmp(computer('arch'),'glnxa64')
  commentSave='t2'; commentHead='t2';
else
  commentSave = input('Comment?: ','s');
  if isempty(commentSave)
    commentHead = [];
  else
    blankIndex = strfind(commentSave, ' ');
    if isempty(blankIndex)
      commentHead = commentSave;
    else
      commentHead = commentSave(1:(blankIndex-1));
    end
  end
end

end

function cDir = getCD

cDir = cd;

switch computer('arch')
  case {'maci64','glnxa64'}
    cdSp = strfind(cDir,'/'); cDir = cDir(cdSp(end)+1:end) ;
  otherwise
    cdSp = strfind(cDir,'\'); cDir = cDir(cdSp(end)+1:end) ;
end

end

function [InitialXY ParameterIndex] = impParForDiv(parameters)

InitialXY = struct; ParameterIndex = struct;

ParameterIndex.free=find(parameters(3,:)~=0);
ParameterIndex.fix=find(parameters(3,:)==0); 
ParameterIndex.min=find(parameters(3,ParameterIndex.free)==2|parameters(3,ParameterIndex.free)==4);
ParameterIndex.max=find(parameters(3,ParameterIndex.free)==3|parameters(3,ParameterIndex.free)==4);
ParameterIndex.normalDist=find(parameters(6,ParameterIndex.free)~=1);
ParameterIndex.expDist=find(parameters(6,ParameterIndex.free)==1);
ParameterIndex.divPar=find(parameters(7,ParameterIndex.free)>1);
InitialXY.xDivNum = parameters(7,ParameterIndex.free);
ParameterIndex.divParExp=find(parameters(6,ParameterIndex.free(ParameterIndex.divPar))==1);

InitialXY.freelimits = parameters(4:5,ParameterIndex.free);
InitialXY.xfree_import = parameters(1:2,ParameterIndex.free); %initial min&max parameters
InitialXY.xfree_import(1:2,ParameterIndex.expDist) = ...
  log(InitialXY.xfree_import(1:2,ParameterIndex.expDist));
InitialXY.xfix = parameters(1,ParameterIndex.fix);%parameters for fixed

end